/*
 * National Semiconductor Corporation Gnu General Public License for 
 *
 *    VTest4Linux
 *
 * (GPL License with Export Notice)
 *
 * Copyright (c) 2001 
 * National Semiconductor Corporation. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted under the terms of the GNU General 
 * Public License as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later version  
 *
 * In addition to the terms of the GNU General Public License, neither 
 * the name of the National Semiconductor Corporation nor the names of 
 * its contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE, 
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE. See the GNU General Public License for more details. 
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (June 22, 2001>) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this file; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 */
 
/*
	05/07/2001 Version 1.0.3 Create file
      02/01/2002 Version 1.0.6 Added error checking

*/
#include <sys/time.h>  
#include <sys/types.h> 
#include <sys/stat.h>
#include <sys/ioctl.h> 
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#define MWINCLUDECOLORS
#ifdef __cplusplus
extern "C" {
#endif
#include "windows.h"
#include "wintern.h"		/* for MwRegisterFdInput*/
#include "wintools.h"
#include "device.h"
#include "wintern.h"		/* for MwSetDesktopWallpaper*/
#ifdef __cplusplus
}
#endif
//===============================
/* These are needed to use the V4L2 driver */
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/videodev.h>  /* Video for Linux Two */
//#include "/cygnus/geode_v4l2/ioctl_geode_v.h"
#include <linux/ioctl_geode_v.h>


int fd_video = -1;
int fd_vout = -1;
geode_general_param_t par;

/* driver wrapper */

/* enable dvip mode */
int dvip(int enable)  
{
      if (ioctl(fd_vout,GEODE_IOCTL_S_DVIP,&enable)==-1) {
           printf("error in enable dvip\n");
           return (-1);
      }
      return 0;
}

/* set control value -- not used */
int video_set_control(unsigned long id,unsigned char value)
{
	struct v4l2_control control;
	control.id = id;
	control.value = value;
	if(ioctl(fd_video, VIDIOC_S_CTRL, &control) == -1) {
	  printf ("error in setting video control\n");
        return (-1);
      }
      return 0;
}

/* set brightness */
int video_brightness(unsigned char bright)
{
     if (video_set_control(V4L2_CID_BRIGHTNESS, bright) == -1){
        printf("error in setting brightness\n");
        return (-1);
      }	
      return 0;
}

/* set contrast */
int video_contrast(unsigned char contrast)
{
	if (video_set_control(V4L2_CID_CONTRAST, contrast)==-1){
          printf("error in setting contrast\n");
          return (-1);
	}
	return 0;		
}

/* set saturation */
int video_saturation(unsigned char sat)
{
	if (video_set_control(V4L2_CID_SATURATION, sat) == -1){
	    printf("error in setting saturation\n");
            return (-1);
	}
	return 0;		
}

/* set hue */
int video_hue(unsigned char hue)
{
	if (video_set_control(V4L2_CID_HUE, hue)==-1){
	   printf("error in setting hue\n");
         return (-1);
	}
	return 0;
}

/* set sharpness */
int video_sharpness(unsigned char sharp)
{
   if(video_set_control(V4L2_CID_WHITENESS, sharp) == -1){
	printf("error in setting sharpness\n");	
	return (-1);
   }
   return 0;
}

/* open video, vout, and realmagic drivers */
int open_devices()
{
      int retval = 0;
      int fd_vbi = -1, fd_realm0=-1;
      fd_video = open("/dev/video", O_RDWR);
      if(fd_video == -1){
        printf("Cannot open /dev/video\n");
        printf("Please use: mknod /dev/video c 81 0 to create it\n");
        retval = -1;
      }
      fd_vout = open("/dev/vout", O_RDWR);
      if(fd_vout == -1) {
         printf("Cannot open /dev/vout\n");
         printf("Please use: mknod /dev/vout c 81 16 to create it");
         retval = -1;
      }
      fd_vbi = open("/dev/vbi", O_RDWR);
      if(fd_vbi == -1) {
         printf("Cannot open /dev/vbi\n");
         printf("Please use: mknod /dev/vbi c 81 224 to create it");
         retval = -1;
      } else
        close(fd_vbi);
      fd_realm0 = open("/dev/realmagic0", O_RDWR);
      if(fd_realm0 == -1) {
           printf("Cannot open /dev/realmagic0\n");
           printf("Please use: mknod /dev/realmagic0 c 122 0  to create it\n");
           retval = -1;
      } else
        close(fd_realm0);
      
      return retval;
}

/* close video and vout drivers */
void close_devices()
{
	close(fd_video);
	close(fd_vout);
}

/* get vout format */
int get_vout_format(int * width, int* height) 
{
	struct v4l2_format fmt;
	if (ioctl(fd_vout,VIDIOC_G_FMT,&fmt)== -1){
         printf("error getting vout format\n");
         return (-1);
      }
	*width = fmt.fmt.pix.width;
	*height = fmt.fmt.pix.height;
      return 0;
}

/* set color key for the overlay */
int set_color_key(unsigned long key, unsigned long mask)
{
	par.p_1 = key;
	par.p_2 = mask;
	if(ioctl(fd_vout, GEODE_IOCTL_S_COLOR_KEY,&par)== -1){
		printf ("error in setting color key\n");
            return (-1);
      }
      return 0;
}	

/* set graphic on video */
#ifdef GEODE_V4L2_105
int set_graphic_on_video(int enable, int color_key_src)
{
	par.p_1 = enable;
	par.p_2 = color_key_src;
	if(ioctl(fd_vout, GEODE_IOCTL_S_GRAPHICS_ON_VIDEO, &par)==-1) {
		printf ("error in setting graphics on video\n");
            return (-1);
      }
	return 0;
}
#endif

/* set alpha windows coordinates */
int set_alpha_window(unsigned char num, unsigned short x, unsigned short y,
			unsigned short width, unsigned short height)
{
	par.p_1 = num;
	par.p_2 = x;
	par.p_3 = y;
	par.p_4 = width;
	par.p_5 = height;
	if (ioctl(fd_vout, GEODE_IOCTL_S_ALPHA_WINDOW, &par)== -1) {
	   printf ("error in settign alpha window\n");
           return (-1);
        }
	return 0;
}

/* deploy alpha window */
int set_alpha_operation(unsigned char num, unsigned char alpha, 
			unsigned long color, int color_enable, 
			int priority, char fade, int enable)
{
	par.p_1 = num;
	par.p_2 = alpha;
	par.p_3 = color;
	par.p_4 = color_enable;
	par.p_5 = priority;
	par.p_6 = fade;
	par.p_7 = enable;
	if(ioctl(fd_vout, GEODE_IOCTL_S_ALPHA_OPER, &par)== -1){
		printf ("error in setting alpha operation\n");
		return (-1);
 	}
	return 0;
}

/* enable video outside of alpha windows */
#ifdef GEODE_V4L2_105
int set_video_outside_alpha(int enable)
{
	par.p_1 = (unsigned long)enable;
	if(ioctl(fd_vout, GEODE_IOCTL_VIDEO_OUTSIDE_ALPHA, &par)== -1) {
	   printf ("error in settign video ouside alpha window\n");
         return (-1);
	}
	return 0;
}
#endif

/* set overlay */
int set_overlay_info( unsigned long key, unsigned long mask, int graphics, 
			int enable)
{
	overlay_info_t ovr_info;
	ovr_info.ColorKey = key;
	ovr_info.ColorMask = mask;
#ifdef GEODE_V4L2_105
	ovr_info.Graphics = graphics;
	ovr_info.Enable = enable;
#endif	
	if(ioctl(fd_vout, GEODE_IOCTL_SET_OVERLAY_INFO, &ovr_info)== -1) {
         printf("error in setting overlay\n");
         return (-1);
	}
	return 0;
}

/* set alpha blend */
int set_alpha_blend_info(int window, int x, int y, int w, int h, 
			  int priority, int alpha, int delta, int enable,
			  unsigned long color)
{
	alpha_blend_info_t alp_info;
	alp_info.WindowNumber = window;
	alp_info.StartX = x;
	alp_info.StartY = y;
	alp_info.Width = w;
	alp_info.Height = h;
	alp_info.Alpha = alpha;
	alp_info.Delta = delta;
	alp_info.Enable = enable;
	alp_info.Color = color;
	if (ioctl(fd_vout, GEODE_IOCTL_SET_ALPHA_BLEND_INFO, &alp_info)==-1){
	   printf("error in setting alpha blend\n");
         return (-1);
	}
	return 0;
}

/* set video format */
int set_video_fmt(int width, int height)
{
	struct v4l2_format fmt;
	fmt.fmt.pix.width = width;
	fmt.fmt.pix.height = height;
	if(ioctl(fd_video, VIDIOC_S_FMT, &fmt)==-1){
	   printf("error in setting video format\n");
         return (-1);
	}
	return 0;
}

/* set vout window coordinates */
int set_vout_window(int x, int y, int width, int height)
{
	struct v4l2_window win;
	win.x = x;
	win.y = y;
	win.width = width;
	win.height = height;
	win.clipcount = 0;
      if(ioctl(fd_vout, VIDIOC_S_WIN, &win)==-1){
	   printf("error in setting vout window\n");
         return (-1);
	}
	return 0;
}

/* select video input */
int set_input(int inp)
{
	int input = inp;
	if( ioctl(fd_video, VIDIOC_S_INPUT,&input)==-1){
	   printf("error in setting video input\n");
         return (-1);
	}
	return 0;
}

/* enable video capturing */
int capture_enable( int enable)
{
	if(enable) {
	   if (ioctl(fd_video, VIDIOC_STREAMON)== -1) {
	      printf("error in enable video capturing\n");
              return (-1);
	   }
	} else {
	    if (ioctl(fd_video, VIDIOC_STREAMOFF) == -1) {
		printf("error in disable video capturing\n");
                return (-1);
	    }
	}
        return 0;
}

/* enable preview */
int preview(int enable)
{
      int i = enable;
      if (ioctl(fd_vout, VIDIOC_PREVIEW,&i) == -1) {
         if(enable)
	     printf("error in enable preveiw\n");
         else
	     printf("error in disable preveiw\n");
           return (-1);
	}
	return 0;
}

/* select tuner channel */
int set_chan(int ch)
{
	unsigned long c = ch;
	if (ioctl(fd_video, VIDIOC_S_FREQ,&c)==-1){
	      printf("error in selecting tuner channel\n");
            return (-1);
	 }
	 return 0;
}

#ifndef GEODE_V4L2_105
int set_vout_control(int val)
{
	int i = 0;
	struct v4l2_control ctrl;
	if (ioctl(fd_vout,VIDIOC_S_EFFECT,&i)== -1) {
	    printf("error in setting S_EFFECT\n");
          return (-1);
	}
	ctrl.id = GEODE_CID_OVERLAY_MODE;
	ctrl.value = val;
	if (ioctl(fd_vout, VIDIOC_S_CTRL, &ctrl)==-1) {
	    printf("error in setting S_CTRL\n");
            return (-1);
	}
	return 0;
}
#endif
/* end driver wrapper */


