#include <stdio.h>
#include <nano-X.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#ifdef SA1100_GRAPHICSCLIENT
#	include "ucb1200_tscal.h"
#else
#	include "h3600_ts.h"
#endif /* SA1100_GRAPHICSCLIENT */

GR_WINDOW_ID w;


typedef struct {
  ushort x;
  ushort y;
} TS_POINT;

static int  xa[5], ya[5];

#define NXCAL_BACKGROUND GR_RGB(0,0,0)
#define NXCAL_FOREGROUND GR_RGB(255,0,0)

#ifdef SA1100_GRAPHICSCLIENT

int rotated = 0;

#if defined TS_EVENT
#undef TS_EVENT
#endif

struct ts_event
{
	short						x,						/* X coordinate value from device */
								y,						/* Y coordinate value from device */
								pressure;				/* Pressure value from device */
} __attribute__((packed));
typedef struct ts_event TS_EVENT;

typedef struct
{
	double						a11,
								a12,
								a13,
								a21,
								a22,
								a23,
								a31,
								a32,
								a33;
} MATRIX;

#define DEV_NODE				"/dev/ts"				/* Device name used by ucb1200 */
#define	SCREEN_WIDTH			640
#define	SCREEN_HEIGHT			480

static int Calibration(long Points, long *ScrX, long *ScrY, long *TchX, long *TchY, CPARM *pCPrm);
static double DetMatrix(MATRIX *pMatrix);

#else
int rotated = 1;
static int calFlag = 0;
static int rawFlag = 0;
static int viewFlag = 0;
static int testFlag = 0;
static int setCalFlag = 0;
static int flashCalFlag = 0;
static int newCalFlag = 0;
static TS_CAL new_cal;
static TS_CAL raw_cal={256, 0, 256, 0 };
static int rate = 50;
#define DEV_NODE "/dev/h3600_tsraw"

/* Note: ITSY geometry screenW=320, screenH=200; H3600 Width=320 Height=240*/
#define SCREEN_WIDTH		320
#define SCREEN_HEIGHT		240
#endif /* SA1100_GRAPHICSCLIENT */
#define SZ_XHAIR                50      /* Num of pixels across xhair */

static void
_drawPlus (int xcoord, int ycoord, int delete)
{
  GR_EVENT event;

  int hx_coord1,hy_coord1,hx_coord2,hy_coord2;
  int vx_coord1,vy_coord1,vx_coord2,vy_coord2;

  GR_GC_ID gc = GrNewGC();

  if (rotated) {
    int temp = xcoord;
    xcoord = ycoord;
    ycoord = SCREEN_WIDTH - temp;
  }
  /* work out the horizontal and vertical line coords */
  
  /* horizontal line */
  hx_coord1= xcoord - (SZ_XHAIR/2);
  hx_coord2= xcoord + (SZ_XHAIR/2);
  hy_coord1=hy_coord2=ycoord;
  /* vertical line */
  vx_coord1=vx_coord2=xcoord;
  vy_coord1= ycoord - (SZ_XHAIR/2);
  vy_coord2= ycoord + (SZ_XHAIR/2);
  

  GrSetGCBackground(gc,NXCAL_BACKGROUND);
  GrSetGCForeground(gc,NXCAL_FOREGROUND);

  if (!delete ) {
    GrSetGCMode(gc,GR_MODE_XOR);
    GrSetGCForeground(gc,NXCAL_FOREGROUND);
  } else {
    GrSetGCMode(gc,GR_MODE_SET);
    GrSetGCForeground(gc,NXCAL_BACKGROUND);
  }
  
  /* draw horizontal line */
  GrLine(w,gc,hx_coord1,hy_coord1,hx_coord2,hy_coord2);
  
  /* draw vertical line */
  GrLine(w,gc,vx_coord1,vy_coord1,vx_coord2,vy_coord2);

  GrDestroyGC(gc);

  while(GrPeekEvent(&event))
    GrGetNextEvent(&event);
}

int last_x = -1;
int last_y = -1;

static void drawPlus(int x, int y, int bogus)
{
  int _x,_y;
  int n = 20;
  int i;

  if(bogus) {
    _drawPlus(_x,_y,1);
    return;
  }


  if(last_x == -1) {
    last_x = x;
    last_y = y;
  }

  if(last_x != x || last_y != y) {

    // erase the previous plus
    _drawPlus(last_x,last_y,0);

    for (i=0; i < n; i++)
      {
	_x = last_x + ((x - last_x) * i / n);
	_y = last_y + ((y - last_y) * i / n);

	_drawPlus(_x,_y,0);
	usleep(60);
	_drawPlus(_x,_y,0);
      }
    last_x = x;
    last_y = y;
  }
    
  _drawPlus(last_x,last_y,0);
}

int g_text_x = 100;

static void
drawText(char * text)
{
  GR_GC_ID gc;
  GR_SIZE tw,th,tb;

  gc = GrNewGC();
  GrSetGCBackground(gc,NXCAL_BACKGROUND);
  GrSetGCForeground(gc,GR_RGB(255,255,255));
  GrSetGCMode(gc,GR_MODE_SET);

  GrGetGCTextSize(gc,text,-1,0,&tw,&th,&tb);

  printf("Doing text at %d, %d\n", 
	 ((rotated ? SCREEN_HEIGHT : SCREEN_WIDTH) - tw)/w, g_text_x);
  
  GrText(w,gc,((rotated ? SCREEN_HEIGHT : SCREEN_WIDTH) - tw)/2, g_text_x,text,-1,0);

  g_text_x += th + 3;

  GrDestroyGC(gc);

}

#define MAX_CAL_POINTS	5

static int 
do_calibration(int fd )
{

  GR_EVENT event;

  int k, xsum, ysum, cnt=0, rv;
  TS_EVENT ts_ev;
  TS_POINT screenCalPos[5];
  int x,y;
  unsigned short xraw,yraw;

#ifdef SA1100_GRAPHICSCLIENT
	long					screenX[MAX_CAL_POINTS] = {320, 50, 50, 590, 590},	/* X "set" coordinates */
							screenY[MAX_CAL_POINTS] = {240, 50, 430, 430, 50},	/* Y "set" coordiantes */
							rawX[MAX_CAL_POINTS] = {0},						/* Raw x values */
							rawY[MAX_CAL_POINTS] = {0};						/* Raw Y values */
	CPARM					sCParm;											/* Calibration matrix */

	memset(&sCParm, 0, sizeof(sCParm));

	/* Reset the kernel's CParm structure */
	if ( (ioctl(fd, 14, &sCParm)) == -1)
	{
		printf("Unable to reset calibration!\n");
	} /* end of if */
#endif /* SA1100_GRAPHICSCLIENT */

#if 0
  /* TODO set what we think is a good calibration*/
  if (ioctl(fd, TS_SET_CAL, &raw_cal) != 0) {
    printf("ERROR: TS_SET_CALIBRATION `raw_cal' ioctl fails!\n");
    return -1;
  }
#endif

/* Screen Offsets  for test calibration */
#define OFF1 30
#define OFF2 50

#ifdef SA1100_GRAPHICSCLIENT
  screenCalPos[0].x = screenX[0];
  screenCalPos[0].y = screenY[0];
  screenCalPos[1].x = screenX[1];
  screenCalPos[1].y = screenY[1];
  screenCalPos[2].x = screenX[2];
  screenCalPos[2].y = screenY[2];
  screenCalPos[3].x = screenX[3];
  screenCalPos[3].y = screenY[3];
  screenCalPos[4].x = screenX[4];
  screenCalPos[4].y = screenY[4];
#else
  screenCalPos[0].x = OFF1;
  screenCalPos[0].y = OFF1;
  screenCalPos[1].x = SCREEN_WIDTH - OFF2 - 1;
  screenCalPos[1].y = OFF2;
  screenCalPos[2].x = OFF2;
  screenCalPos[2].y = SCREEN_HEIGHT - OFF2 - 1;
  screenCalPos[3].x = SCREEN_WIDTH - OFF1 - 1;
  screenCalPos[3].y = SCREEN_HEIGHT - OFF1 - 1;
  screenCalPos[4].x = SCREEN_WIDTH / 2;
  screenCalPos[4].y = SCREEN_HEIGHT / 2;

#endif /* SA1100_GRAPHICSCLIENT */
/*
 *   Enter here with ts events pertaining to screen coords screenCalPos[k]
 */


  drawText("Touch the center of");
  drawText("each cross to calibrate");
  drawText("the screen");

  /*
   * Why this works, when the X server is also trying to read samples,
   * is beyond me.  But we'll take it. - Jim Gettys
   */

  for (k = 0; k < MAX_CAL_POINTS ; k++) 
  {
    /*
     * for each screen position take the average of 5 points.
     */
    
    drawPlus(screenCalPos[k].x, screenCalPos[k].y,0);

#ifndef SA1100_GRAPHICSCLIENT
    cnt = xsum = ysum = 0;
#endif
    
    /* now loop until we have 5 samples - */
    while (1) {
      /*
       * This read() call will block until the user taps the screen
       * upon which it will proceed to gather samples until the pen
       * is lifted. It takes the average of the 'cnt' samples.
       * It will not accept any less than 5 samples.
       */

      if ((rv = read(fd, &ts_ev, sizeof(TS_EVENT))) == -1) {
	usleep(100);
	//	drawPlus(screenCalPos[k].x, screenCalPos[k].y,1);
	continue;
      }
      else if (rv != sizeof(TS_EVENT)) {
	usleep(100);
	//	drawPlus(screenCalPos[k].x, screenCalPos[k].y,1);
	continue;
      }

      /* delete the plus */
      //      drawPlus(screenCalPos[k].x, screenCalPos[k].y,1);
      
      /*
	Dont exit the while loop until we have 5 events
	AND the PEN is UP denoted by pressure=0
	*/

      if (ts_ev.pressure == 0  &&  cnt > 5)
      {
		break;
      }
      else
      {
	  /* accumulate the x coords */
	  if (ts_ev.x > -1 && ts_ev.y > -1)
	  {
	  	xsum += ts_ev.x;
	  	ysum += ts_ev.y;
#ifdef SA1100_GRAPHICSCLIENT
		rawX[cnt] = ts_ev.x;
		rawY[cnt] = ts_ev.y;
		cnt++;
		break;
#endif /* SA1100_GRAPHICSCLIENT */
	  	/* increment the event counter */
	  	cnt++;
      }
	 }
    } /* endwhile */

    /*
     *	Enter here with - agregate of the x,y coords stored in xsum & ysum.
     */


    /* take the average of the x & y points */
    xa[k] = xsum/cnt;
    ya[k] = ysum/cnt;
  }

  /* Enter here with the average x,y coords of the MAX_CAL_POINTS */

#ifdef SA1100_GRAPHICSCLIENT
	Calibration(MAX_CAL_POINTS, screenX, screenY, rawX, rawY, &sCParm);
	if (sCParm.delta > 1)
	{
		if ( (ioctl(fd, 13, &sCParm)) == -1)
		{
			printf("Error setting calibration matrix!\n");
			return (-1);
		} /* end of if */
	} /* end of if */
#else
  /* get calibration parameters */
  {
    int *xp, *yp, x, y, dx, dy, flag;
    int xscale0, xscale1, xtrans0, xtrans1;
    int yscale0, yscale1, ytrans0, ytrans1;
    int xyswap;
    
    flag = 0;
  
    /* Calculate ABS(x1-x0) */
    dx = ((x = xa[1] - xa[0]) < 0 ? -x : x);
    dy = ((y = ya[1] - ya[0]) < 0 ? -y : y);

    /* CF

	(0,0) --------->Y
X	+-------+---------------+-
|	|	|		|
V	|	|xa0,ya0	|
	+-------O---------------+ --
	|	|		| dx or dy
	+-------+---------------O --
^	|	|		| xa1,ya1
|	|	|		|
Y	+-------+---------------+
	(0,0) -------->X

	Work out where the origin is, either at the TLH or BLH corners.
        Initialise xp such that it points to the array containing the X coords
        Initialise yp such that it points to the array containing the Y coords
    */
    if (dx < dy) {
      xyswap = 1;
      xp = ya, yp = xa;
    }
    else {
      xyswap = 0;
      xp = xa;
      yp = ya;
    }
    
    xraw=xp[4]; yraw=yp[4];	/* used for check later */

    /*

	We have MAX_CAL_POINTS sets of x,y coordinates.

	If we plot Xcal against Xraw we get two equations, each of a straight
		line. One for the Xcoord and the other for the Y coord.
		This line models the linear characteristics of the ts A/D
		converter.

	Xcal = m*Xraw + Cx
	Ycal = m*Yraw + Cy

	X/Ycal is the calibrated coord which is the pixel pos on the screen.
	X/Yraw is the uncalibrated X coord.
	m is the xscale ( gradient of line)
	Cx/y is the trans (constant)
	
	xscale
	  'm' can be got by calculating the gradient between two data points
	  Example Xscale0 = (Xcal1 - Xcal0 ) / (Xraw1 - Xraw0)

	trans = Xcal - mXraw
	  What is actualy done is to take the Ave of two measurements
	  Example  Xtrans0 = ( (Xcal0 - mXraw0) + (Xcal3 - mXraw3) ) / 2

	We repeat the above procedure to calculate 
	Yscale0 and Ytrans0 and repeat the whole lot again using two
	new data indexes 1 and 2 giving 4 new variables
	Xscale1, Xtrans1, Yscale1,Ytrans1, making a total of eight.

	The final calibration variables are the average of data ponts 
		0,3 and 1,2
	xscale = (Xscale0 + Xscale1) / 2
	yscale = (Yscale0 + Yscale1) / 2
	xtrans = (Xtrans0 + Xtrans1) /2
	ytrans = (Ytrans0 + Ytrans1) /2
	
    */
    xscale0 = ( (screenCalPos[0].x - screenCalPos[3].x) << 8 ) / 
      (((xp[0] - xp[3])));

/*
    printf("Xc0=%d Xc3=%d Xr0=%d Xr3=%d xscale0=%d\n",
    	screenCalPos[0].x, screenCalPos[3].x, xp[0],xp[3],xscale0 );
*/
    xtrans0 = ( (screenCalPos[0].x - ((xp[0]*xscale0) >> 8)) + 
		(screenCalPos[3].x - ((xp[3]*xscale0) >> 8)) ) / 2;
    yscale0 = ( (screenCalPos[0].y - screenCalPos[3].y) << 8 ) / 
      (((yp[0] - yp[3])));
    ytrans0 = ( (screenCalPos[0].y - ((yp[0]*yscale0) >> 8)) + 
		(screenCalPos[3].y - ((yp[3]*yscale0) >> 8)) ) / 2;

    xscale1 = ( (screenCalPos[1].x - screenCalPos[2].x) << 8 ) / 
      (((xp[1] - xp[2])));
/*
    printf("Xc1=%d Xc2=%d Xr1=%d Xr2=%d xscale1=%d\n",
    	screenCalPos[1].x, screenCalPos[2].x, xp[1],xp[2],xscale1 );
*/
    xtrans1 = ( (screenCalPos[1].x - ((xp[1]*xscale1) >> 8)) +
		(screenCalPos[2].x - ((xp[2]*xscale1) >> 8)) ) / 2;
    yscale1 = ( (screenCalPos[1].y - screenCalPos[2].y) << 8 ) / 
      (((yp[1] - yp[2])));
    ytrans1 = ( (screenCalPos[1].y - ((yp[1]*yscale1) >> 8)) 
		+(screenCalPos[2].y - ((yp[2]*yscale1) >> 8)) ) / 2;
/*    
    printf("xs0:%d, xs1:%d, xt0:%d, xt1:%d\n", xscale0, xscale1,
	      xtrans0, xtrans1);
    printf("ys0:%d, ys1:%d, yt0:%d, yt1:%d\n", yscale0, yscale1,
	      ytrans0, ytrans1);
*/
    new_cal.xscale = (xscale0 + xscale1) / 2;
    //    printf("AveXscale=%d\n",new_cal.xscale);
    new_cal.xtrans = (xtrans0 + xtrans1) / 2;
    new_cal.yscale = (yscale0 + yscale1) / 2;
    new_cal.ytrans = (ytrans0 + ytrans1) / 2;
    new_cal.xyswap = xyswap;
  }

/*
    printf("New calibration:\n");
    printf("  xscale:%5d, xtrans:%5d\n", new_cal.xscale,
	    new_cal.xtrans);
    printf("  yscale:%5d, xtrans:%5d\n", new_cal.yscale,
	    new_cal.ytrans);
    printf("xyswap:%s\n", (new_cal.xyswap == 0 ? "N" : "Y"));
*/
  /* Now check it with center coords*/
  //  printf("CHECK with Center Coords (160,120): xraw=%d yraw=%d\n",xraw,yraw);
  x = ( ( new_cal.xscale * xraw ) >> 8 ) + new_cal.xtrans;
  y = ( ( new_cal.yscale * yraw ) >> 8 ) + new_cal.ytrans;
  //  printf("CHECK: x(center)=%d y(center)=%d\n",x,y);

  /* store this calibration in the device */
#if 1
  if (ioctl(fd, TS_SET_CAL, (void *)&new_cal) != 0)
  {
    perror("TS_SET_CALIBRATION ioctl fail\n");
    return -1;
  }
#endif
#endif /* SA1100_GRAPHICSCLIENT */


  return 0;
}

int initX()
{
  GrOpen();

  w = GR_ROOT_WINDOW_ID;
  
  return 0;
}


#ifndef SA1100_GRAPHICSCLIENT
void turn_bl_on()
{
   int             fd;
   FLITE_IN        bl;

   fd = open("/dev/h3600_ts", O_RDWR);
   if (fd == -1)
   return;

   bl.mode = 2;
   bl.pwr = 1;
   bl.brightness = 255;
   ioctl(fd, FLITE_ON, (void *) &bl);

   close(fd);
}
#endif /* !SA1100_GRAPHICSCLIENT */

int main(int argc, char **argv)
{
  int k, fd,err;
#ifndef SA1100_GRAPHICSCLIENT
  LED_IN led_on = { 1,1,1,1};		/* blink the led */
  LED_IN led_off = { 0,0,0,0};		/* blink the led */
  unsigned brightness;			/* set by user */
#endif

  if( initX() < 0 )
  {
    	printf("Unable to Xinit\n");
	exit(1);
  }

#ifndef SA1100_GRAPHICSCLIENT
  for (k=1; k<argc; k++) {
    if (!strcmp(argv[k], "-raw"))
      rawFlag = 1, calFlag = 0;
    else if (!strcmp(argv[k], "-view"))
      viewFlag = 1, calFlag = 0;
    else if (!strcmp(argv[k], "-test"))
      testFlag = 1, calFlag = 0;
    else if (!strcmp(argv[k], "-rate")) 
      sscanf(argv[++k], "%d", &rate);
    else if (!strcmp(argv[k], "-cal")) {
      sscanf(argv[++k], "%d", &new_cal.xscale);
      sscanf(argv[++k], "%d", &new_cal.xtrans);
      sscanf(argv[++k], "%d", &new_cal.yscale);
      sscanf(argv[++k], "%d", &new_cal.ytrans);
      sscanf(argv[++k], "%d", &new_cal.xyswap);
      setCalFlag = 1, calFlag = 0;
    }
    else if (!strncmp(argv[k], "-h", 2)) {
      printf("USAGE: tstest <optional arguments>\n");
      printf("  optional args:\n");
      printf("    -view	       view ts calibrated data\n");
      printf("    -raw         view ts raw (non-calibrated) data\n");
      printf("    -test        show + where screen is pressed\n");
      printf("    -rate <rate> set digitazing rate (per sec)\n");
      printf("    -cal <xs> <xt> <ys> <yt> <xyswap>\n");
      printf("                 set new session calibration\n");
      exit(0);
    }
  }


	/* Turn on the backlight so the user can see what he is doing */
	turn_bl_on();
#endif /* !SA1100_GRAPHICSCLIENT */
	
    if ((fd = open(DEV_NODE, O_RDONLY)) < 0) 
    {
	printf("ERROR: unable to open %s",DEV_NODE);
	exit(1);
    }

#if 0

    /* TODO - SET/GET RATE */
    if (ioctl(ts_fd, TS_GET_RATE, &orig_rate) != 0)
    {
	printf("ERROR: TS_GET_RATE ioctl fails!\n");
	close(ts_fd);
	exit(1);
    }

    if (ioctl(ts_fd, TS_SET_RATE, &rate) != 0)
    {
	printf("ERROR: TS_SET_RATE ioctl fails!\n");
	close(ts_fd);
	exit(1);
    }

    /* TODO - store calibration in flash */
    ioctl(ts_fd, TS_SET_CALIBRATION, &new_cal);

#endif


  do_calibration (fd);

  /* close session devices */
  (void) close(fd);

  /* everything OK */
  return 1;
}

#ifdef SA1100_GRAPHICSCLIENT

/*
** Algorithm taken from Woojung Huh's calibration utility <whuh@applieddata.net>
*/
static int
Calibration(long Points, long *ScrX, long *ScrY, long *TchX, long *TchY, CPARM *pCPrm)
{
	int						i;
	double					a1,
							a11,
							a2,
							a21,
							a22,
							a31,
							a32,
							a33,
							b1,
							b11,
							b12,
							b13,
							b2,
							b21,
							b22,
							b23,
							c1,
							c2,
							delta,
							TempScrX,
							TempScrY,
							TempTchX,
							TempTchY;
	MATRIX					Matrix;

	if (Points == 0)
		return (0);

	a11 = a21 = a31 = a22 = a32 = b11 =
	b12 = b13 = b21 = b22 = b23 = 0;

	for (i = 0; i < Points; i++)
	{
		TempTchX = (double) TchX[i];
		TempTchY = (double) TchY[i];
		TempScrX = (double) ScrX[i];
		TempScrY = (double) ScrY[i];
		a11 += TempTchX * TempTchX;
		a21 += TempTchX * TempTchY;
		a31 += TempTchX;
		a22 += TempTchY * TempTchY;
		a32 += TempTchY;
		b11 += TempTchX * TempScrX;
		b12 += TempTchY * TempScrX;
		b13 += TempScrX;
		b21 += TempTchX * TempScrY;
		b22 += TempTchY * TempScrY;
		b23 += TempScrY;
	} /* end of for */
	a33 = Points;

	Matrix.a11 = a11;
	Matrix.a21 = a21;
	Matrix.a31 = a31;
	Matrix.a12 = a21;
	Matrix.a22 = a22;
	Matrix.a32 = a32;
	Matrix.a13 = a31;
	Matrix.a23 = a32;
	Matrix.a33 = a33;
	delta = DetMatrix(&Matrix);

	Matrix.a11 = b11;
	Matrix.a21 = b12;
	Matrix.a31 = b13;
	a1 = DetMatrix(&Matrix);

	Matrix.a11 = a11;
	Matrix.a21 = a21;
	Matrix.a31 = a31;
	Matrix.a12 = b11;
	Matrix.a22 = b12;
	Matrix.a32 = b13;
	b1 = DetMatrix(&Matrix);

	Matrix.a12 = a21;
	Matrix.a22 = a22;
	Matrix.a32 = a32;
	Matrix.a13 = b11;
	Matrix.a23 = b12;
	Matrix.a33 = b13;
	c1 = DetMatrix(&Matrix);

	Matrix.a13 = a31;
	Matrix.a23 = a32;
	Matrix.a33 = a33;
	Matrix.a11 = b21;
	Matrix.a21 = b22;
	Matrix.a31 = b23;
	a2 = DetMatrix(&Matrix);

	Matrix.a11 = a11;
	Matrix.a21 = a21;
	Matrix.a31 = a31;
	Matrix.a12 = b21;
	Matrix.a22 = b22;
	Matrix.a32 = b23;
	b2 = DetMatrix(&Matrix);

	Matrix.a12 = a21;
	Matrix.a22 = a22;
	Matrix.a32 = a32;
	Matrix.a13 = b21;
	Matrix.a23 = b22;
	Matrix.a33 = b23;
	c2 = DetMatrix(&Matrix);

	pCPrm->a1 = (long) (a1 / 1000000.0);
	pCPrm->b1 = (long) (b1 / 1000000.0);
	pCPrm->c1 = (long) (c1 / 1000000.0);
	pCPrm->a2 = (long) (a2 / 1000000.0);
	pCPrm->b2 = (long) (b2 / 1000000.0);
	pCPrm->c2 = (long) (c2 / 1000000.0);
	pCPrm->delta = (long) (delta / 1000000.0);

	if (pCPrm->delta == 0)
		pCPrm->delta = 1;

	return 1;
} /* end of Calibration() */

static double
DetMatrix(MATRIX *pMatrix)
{
	return ((pMatrix->a11 * pMatrix->a22 * pMatrix->a33) +
			(pMatrix->a13 * pMatrix->a21 * pMatrix->a32) +
			(pMatrix->a12 * pMatrix->a23 * pMatrix->a31) -
			(pMatrix->a13 * pMatrix->a22 * pMatrix->a31) - 
			(pMatrix->a12 * pMatrix->a21 * pMatrix->a33) - 
			(pMatrix->a23 * pMatrix->a32 * pMatrix->a11));
} /* end of DetMatrix */

#endif /* SA1100_GRAPHICSCLIENT */
