/*
 * Window Manager/Screentop for Nano-X
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 * Copyright (C) 2000 Greg Haerr <greg@censoft.com>
 * Copyright (C) 2000 Alex Holden <alex@linuxhacker.org>
 */
#include <stdio.h>
#include <stdlib.h>
#define MWINCLUDECOLORS
#include "nano-X.h"
#include "nxdraw.h"
/* Uncomment this if you want debugging output from this file */
//#define DEBUG
#include "nanowm.h"
#include "apps.h"

//#define OUTLINE_MOVE 1
//#define OUTLINE_RESIZE 1

#ifdef SYSTEM_MENUS
#include "sys_menu.h"

static void container_close(void *window)
{
  if (!window)
    return;

  GrCloseWindow(((win *)window)->clientid);
}

static wm_menu_t container_sys_menu[] = {
{ "(Un)Stick", MENU_TYPE_REGULAR, 0},
{ "Close", MENU_TYPE_REGULAR, container_close},
{ "", MENU_TYPE_END, 0}};

#endif

static void
container_redraw(win *window)
{
	GR_WINDOW_INFO info;
	GR_WM_PROPERTIES props;
	GR_BOOL active;

	Dprintf("container_exposure window %d\n", window->wid);

	GrGetWindowInfo(window->wid, &info);
	GrGetWMProperties(window->clientid, &props);

	/*
	 * Check for invalid window.  This will be the
	 * case if the client exited, and we're just
	 * getting the paint notification for our parent.
	 */
	if (props.flags == 0)
		return;

	active = (window->clientid == GrGetFocus());
	nxPaintNCArea(window->wid, info.width, info.height, props.title,
		active, props.props);

	/* Ack, this was malloced, and it is our responsibility to free it */
	if (props.title)
	  free(props.title);
}

static void
container_exposure(win *window, GR_EVENT_EXPOSURE *event)
{
	Dprintf("container_exposure window %d\n", window->wid);

	container_redraw(window);
}

static GR_BOOL
PtInRect(GR_RECT *prc, GR_SIZE x, GR_SIZE y)
{
	return (x >= prc->x && x < (prc->x+prc->width) &&
		y >= prc->y && y < (prc->y+prc->height));
}

static void
container_buttondown(win *window, GR_EVENT_BUTTON *event)
{
	GR_RECT		r;
	GR_COORD	cxborder = 0, cyborder = 0;
	GR_WINDOW_INFO	info;

	struct pos_size * pos = &window->pos;

	Dprintf("container_buttondown window %d\n", window->wid);

	if(window->mousedn)
		return;

	GrGetWindowInfo(window->wid, &info);

	/* calc border sizes*/
	if (info.props & GR_WM_PROPS_BORDER) {
		cxborder = 1;
		cyborder = 1;
	}
	if (info.props & GR_WM_PROPS_APPFRAME) {
		cxborder = CXBORDER;
		cyborder = CYBORDER;
	}

	/* Check for close box press*/
	if ((info.props & (GR_WM_PROPS_CAPTION|GR_WM_PROPS_CLOSEBOX)) ==
	    (GR_WM_PROPS_CAPTION|GR_WM_PROPS_CLOSEBOX)) {

		/* Get close box rect*/
		r.x = info.width - CXCLOSEBOX - cxborder - 2;
		r.y = cyborder + 2;
		r.width = CXCLOSEBOX;
		r.height = CYCLOSEBOX;

		/* Check mousedn in close box*/
		if (PtInRect(&r, event->x, event->y)) {
			/* this may or not close the window...*/
			GrCloseWindow(window->clientid);
			return;
		}
	}

	/* Set focus on button down*/
	GrSetFocus(window->clientid);

	/* check for corner resize */
	r.x = info.width - 5;
	r.y = info.height - 5;
	r.width = 5;
	r.height = 5;

	if((info.props & GR_WM_PROPS_APPFRAME)
	   && PtInRect(&r,event->x, event->y)) {

	  window->sizing = GR_TRUE;
	  pos = &window->pos;
	  
	  // save off the width/height offset from the window manager
	  GrGetWindowInfo(window->clientid,&info);
	  pos->xoff = -info.width;
	  pos->yoff = -info.height;

	  GrGetWindowInfo(window->wid,&info);
	  pos->xoff += info.width;
	  pos->yoff += info.height;


#ifdef OUTLINE_RESIZE
	  // let mouse move know that we havn't drawn the rect yet
	  pos->width = -1;
	  pos->height = -1;
#endif
	  window->mousedn = GR_TRUE;
	  return;
	}

	/* if not in caption, return (FIXME, not calc'd exactly)*/
	if (!(info.props & GR_WM_PROPS_CAPTION))
		return;

	/* Get caption box rect*/
	r.x = cxborder;
	r.y = cyborder;
	r.width = info.width - cxborder*2;
	r.height = CYCAPTION;

	/* Check for mousedn in caption box*/
	if (!PtInRect(&r, event->x, event->y))
		return;

#ifdef SYSTEM_MENUS
	
	/* If it was a right click, then bring up the system menu */

	if (event->buttons & GR_BUTTON_R)
	  {
	    container_sys_menu[1].cbdata = (void *) window;
	    open_system_menu(window->wid, event->x, event->y, 
			     MENU_DIR_DOWN, container_sys_menu);
	    
	    return;
	  }
#endif
	
	/* Raise window if mouse down and allowed*/
	if (!(info.props & GR_WM_PROPS_NORAISE))
		GrRaiseWindow(window->wid);

	/* Don't allow window move if NOMOVE property set*/
	if (info.props & GR_WM_PROPS_NOMOVE)
		return;

	window->x = event->x;
	window->y = event->y;

      	pos->xoff = event->x;
	pos->yoff = event->y;

#ifdef OUTLINE_MOVE
	pos->xorig = -1;
	pos->yorig = -1;
	pos->width = info.width;
	pos->height = info.height;
#endif	
	window->mousedn = GR_TRUE;
}

static void
container_buttonup(win *window, GR_EVENT_BUTTON *event)
{
#if OUTLINE_RESIZE | OUTLINE_MOVE
	GR_GC_ID gc;
#endif
	struct pos_size * pos = &window->pos;

	Dprintf("container_buttonup window %d\n", window->wid);
	
	if(window->mousedn && !window->sizing &&
	   pos->xorig != -1 && pos->yorig != -1) {
#ifdef OUTLINE_MOVE
	  gc = GrNewGC();
	  GrSetGCMode(gc, GR_MODE_XOR|GR_MODE_EXCLUDECHILDREN);
	  GrRect(GR_ROOT_WINDOW_ID,gc,pos->xorig, pos->yorig,
		 pos->width, pos->height);
	  GrMoveWindow(window->wid, pos->xorig, pos->yorig);

#endif
	} else if(window->sizing && pos->width != -1 && pos->height != -1) {
	  GR_WINDOW_INFO info;

	  GrGetWindowInfo(window->wid, &info);

#ifdef OUTLINE_RESIZE
	  gc = GrNewGC();
	  GrSetGCMode(gc, GR_MODE_XOR|GR_MODE_EXCLUDECHILDREN);

	  GrRect(GR_ROOT_WINDOW_ID,gc,info.x, info.y, pos->width, pos->height);
	  GrDestroyGC(gc);
#endif
	  GrResizeWindow(window->wid,event->rootx - info.x,
			  event->rooty - info.y);
	}

	window->sizing = GR_FALSE;
	window->mousedn = GR_FALSE;
}

static void
container_mousemoved(win *window, GR_EVENT_MOUSE *event)
{
  GR_WINDOW_INFO info;
  struct pos_size * pos;
#if OUTLINE_RESIZE | OUTLINE_MOVE
  GR_GC_ID gc;
#endif
	Dprintf("container_mousemoved window %d\n", window->wid);

	if(!window->mousedn)
		return;

	pos = (struct pos_size *) &window->pos;

	if(window->sizing) {

	  GrGetWindowInfo(window->wid, &info);

#ifdef OUTLINE_RESIZE
	  gc = GrNewGC();
	  GrSetGCMode(gc, GR_MODE_XOR|GR_MODE_EXCLUDECHILDREN);

	  // erase old rectangle if necessary
	  if(pos->width != -1 && pos->height != -1) 
	    GrRect(GR_ROOT_WINDOW_ID,gc,info.x, info.y, pos->width, pos->height);

	  // draw new one
	  GrRect(GR_ROOT_WINDOW_ID,gc,info.x, info.y, 
		 event->rootx - info.x, event->rooty - info.y);

	  GrDestroyGC(gc);
	  // save this new rectangle's width, height
	  // I know, this shouldn't be stored in x/y, but...
	  pos->width = event->rootx - info.x;
	  pos->height = event->rooty - info.y;
#else
	  GrResizeWindow(window->wid,event->rootx - info.x,
		  event->rooty - info.y);
#endif
	  return;
	}

#ifdef OUTLINE_MOVE
	gc = GrNewGC();
	GrSetGCMode(gc, GR_MODE_XOR|GR_MODE_EXCLUDECHILDREN);

	if(pos->xorig != -1 && pos->yorig != -1)
		GrRect(GR_ROOT_WINDOW_ID,gc,pos->xorig, pos->yorig,
			  pos->width, pos->height);

	GrRect(GR_ROOT_WINDOW_ID,gc,event->rootx - pos->xoff,
		event->rooty - pos->yoff, pos->width, pos->height);

	pos->xorig = event->rootx - pos->xoff;
	pos->yorig = event->rooty - pos->yoff;
	
	GrDestroyGC(gc);
#else	
	GrMoveWindow(window->wid, event->rootx - pos->xoff,
		event->rooty - pos->yoff);
#endif

#ifdef VIRTUAL_WINDOWS
	/* Redraw the window box so it knows whats up */
	/* virtswitch_redraw(); */
#endif
}

static void
container_child_update(win *window, GR_EVENT_UPDATE *ep)
{
	switch(ep->utype) {
	case GR_UPDATE_ACTIVATE:
		container_redraw(window);
		break;
	}
}

static void 
container_resize(win * window, GR_EVENT_UPDATE * ep)
{
	int w,h;

	GR_WINDOW_INFO info;
	GrGetWindowInfo(window->wid,&info);

	// calculate new size for the client window
	nxCalcClientSize(info.props,ep->width,ep->height,NULL,NULL,&w,&h);
	// resize client window
	GrResizeWindow(window->clientid,w,h);
}

static void
container_update(win *window, GR_EVENT_UPDATE *ep)
{
	switch(ep->utype) {
	case GR_UPDATE_SIZE:
		container_resize(window,ep);
		break;
	}
}

void
container_wndproc(win *window, GR_EVENT *ep)
{
	switch(ep->type) {
	case GR_EVENT_TYPE_EXPOSURE:
		container_exposure(window, &ep->exposure);
		break;
	case GR_EVENT_TYPE_BUTTON_DOWN:
		container_buttondown(window, &ep->button);
		break;
	case GR_EVENT_TYPE_BUTTON_UP:
		container_buttonup(window, &ep->button);
		break;
	case GR_EVENT_TYPE_UPDATE:
	  container_update(window,&ep->update);
	  break;
	case GR_EVENT_TYPE_MOUSE_POSITION:
		container_mousemoved(window, &ep->mouse);
		break;
	case GR_EVENT_TYPE_CHLD_UPDATE:
		container_child_update(window, &ep->update);
		break;
	}
}
