/*
 * Window Manager/Screentop for Nano-X
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 *
 * Written by Jason Kingan and Greg Haerr
 */
#include <stdio.h>
#include "nano-X.h"
#include "apps.h"
#include "nanowm.h"
#include "catagories.h"

#define HOMEICON "home.gif"

GR_BOOL g_clear_window = 0;

static GR_IMAGE_ID imageid;

static void home_wndproc(win *window, GR_EVENT *ep);

void 
home_create(void)
{
	GR_WINDOW_ID	wid;
	GR_SCREEN_INFO	si;

	GrGetScreenInfo(&si);
	wid = GrNewWindowEx(GR_WM_PROPS_NODECORATE, NULL, GR_ROOT_WINDOW_ID,
		si.ws_width-60, si.ws_height + 2, 20, 20, COLOR_TASKBAR);
	add_window(wid, GR_ROOT_WINDOW_ID, 0, home_wndproc);

	GrSelectEvents(wid, GR_EVENT_MASK_BUTTON_DOWN | GR_EVENT_MASK_EXPOSURE);
	GrMapWindow(wid);
}

static void 
home_exposure(win *window, GR_EVENT_EXPOSURE *ep)
{
  GR_GC_ID gc = GrNewGC();
  
  if (!imageid)
    {
      char path[256];      
     
      sprintf(path, "%s/%s", 
	      get_desktop_config(DESKTOP_IMAGEDIR), HOMEICON);
      imageid = GrLoadImageFromFile(path, 0);
    }
  
  GrDrawImageToFit(ep->wid, gc, 0, 0, -1, -1, imageid);
  GrDestroyGC(gc);
}

static void 
home_buttondown(win *window, GR_EVENT_BUTTON *ep)
{
	if (g_clear_window && g_last_window) {
		apps_buttondown(g_last_window);
		g_clear_window = 0;
		return;
	}

#ifdef VIRTUAL_WINDOWS
	app_hide_windows(get_current_virtwin());
#else
	app_hide_windows();
#endif

	g_clear_window = 1;
}

static void
home_wndproc(win *window, GR_EVENT *ep)
{
	switch(ep->type) {
	case GR_EVENT_TYPE_EXPOSURE:
		home_exposure(window, &ep->exposure);
		break;
	case GR_EVENT_TYPE_BUTTON_DOWN:
		home_buttondown(window, &ep->button);
		break;
	}
}
