/*
 * Window Manager/Screentop for Nano-X
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 * Copyright (C) 2000 Greg Haerr <greg@censoft.com>
 * Copyright (C) 2000 Alex Holden <alex@linuxhacker.org>
 */

#ifdef DEBUG
#define Dprintf printf
#else
#define Dprintf(ignore...)
#endif

typedef struct windowlist *winptr;
typedef void (*wndproc)(struct windowlist *window, GR_EVENT *ep);

struct pos_size {
	GR_COORD xoff;
	GR_COORD yoff;
	GR_COORD xorig;
	GR_COORD yorig;
	GR_SIZE width;
	GR_SIZE height;
};

/* all root-mapped windows are kept track of with this structure*/
struct windowlist {
	GR_WINDOW_ID wid;	/* The ID of this window */
	GR_WINDOW_ID pid;	/* The ID of this window's parent */
	GR_WINDOW_ID clientid;	/* clientid for container window*/
	GR_BOOL	mousedn;	/* TRUE when mouse is down in window*/
	GR_COORD x;		/* mouse down coordinates*/
	GR_COORD y;
	wndproc proc;		/* callback associated with window*/
	struct windowlist *next; /* The next window in the list */
	int sizing;		/* used during resize handling*/
	struct pos_size pos;	/* used during resize/move handling*/
};
typedef struct windowlist win;

/* wlist.c*/
win *find_window(GR_WINDOW_ID wid);
win *add_window(GR_WINDOW_ID wid, GR_WINDOW_ID pid, GR_WINDOW_ID clid,
	wndproc proc);
int remove_window(win *window);
int remove_window_and_children(win *window);

/* main.c*/
void timeout_setproc(wndproc proc,win *window);

/* container.c*/
void container_wndproc(win *window, GR_EVENT *ep);

/* clients.c*/
int client_updatemap_new(GR_WINDOW_ID wid);

/* apps.c */
void apps_free_memory();

/* root.c */
void root_free_memory();

/* icons.c */
void hide_icon_list(void);
void draw_current_iconlist(void);
GR_WINDOW_ID create_icon_window();

/* catagories.c */
void catagory_free_memory();

/* settings.c */

#define DESKTOP_PREFIX   0
#define DESKTOP_IMAGEDIR 1
#define DESKTOP_NXAPPDIR 2
#define DESKTOP_SOUNDDIR 3

void load_desktop_paths(char *filename);
char* get_desktop_config(int flag);
