#ifdef VIRTUAL_WINDOWS

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MWINCLUDECOLORS
#include "nano-X.h"
#include "apps.h"
#include "nanowm.h"
#include "catagories.h"
#include "settings.h"

GR_WINDOW_ID cswid;
GR_WINDOW_ID cat_pixmap;

GR_SCREEN_INFO si;

void draw_catswitch(GR_RECT *rc)
{
  int width, height, baseline, x, y;
  CATAGORY *cat = get_current_catagory();  
  GR_WINDOW_INFO winfo;
  GR_GC_ID gc;
  GR_GC_ID pgc;
  GR_FONT_ID fnt;
  GR_POINT points[3];

	/* Only display category windows if more than 1 category */
  if (get_catagory_count() < 2)
	  return;

  GrGetWindowInfo(cswid, &winfo);

  if ((rc->x + rc->width) < winfo.x || rc->x > winfo.x + winfo.width)
    return;
  
  if ((rc->y + rc->height) < winfo.y || rc->y > winfo.y + winfo.height)
    return;

  fnt = GrCreateFont(GR_FONT_GUI_VAR, 0, NULL);
  gc = GrNewGC();
  pgc = GrNewGC();
  
  GrSetGCForeground(gc, wm_get_color(WM_BGCOLOR));
  GrFillRect(GR_ROOT_WINDOW_ID, gc, winfo.x, winfo.y, winfo.width, winfo.height);

  /* Now, in the pixmap, create the window */

  GrSetGCForeground(pgc, LTGRAY);
  GrFillRect(cat_pixmap, pgc, 0, 0, winfo.width, winfo.height);

  GrSetGCForeground(pgc, BLACK);
  GrRect(cat_pixmap, pgc, 0, 0, winfo.width, winfo.height);

  points[0].x = 1;
  points[0].y = winfo.height / 2;
  points[1].x = 10;
  points[1].y = 1;
  points[2].x = 10;
  points[2].y = winfo.height - 2;

  GrFillPoly(cat_pixmap, pgc, 3, points);

  points[0].x = winfo.width - 2;
  points[0].y = winfo.height / 2;
  points[1].x = winfo.width - 10;
  points[1].y = 1;
  points[2].x = winfo.width - 10;
  points[2].y = winfo.height - 2;

  GrFillPoly(cat_pixmap, pgc, 3, points);

#ifdef USE_ALPHABLEND

  GrCopyArea(GR_ROOT_WINDOW_ID, gc, winfo.x, winfo.y, winfo.width, 
	     winfo.height,
	     cat_pixmap, 0, 0, MWROP_BLENDCONSTANT | 100);

#else

GrCopyArea(GR_ROOT_WINDOW_ID, gc, winfo.x, winfo.y, winfo.width, 
	     winfo.height,
	     cat_pixmap, 0, 0, MWROP_SRCCOPY);

#endif

 
  GrSetGCForeground(gc, BLACK);
  GrSetGCUseBackground(gc, GR_FALSE);

  GrSetGCFont(gc, fnt);

  GrGetGCTextSize(gc, cat->title, -1,
		  GR_TFASCII|GR_TFTOP, &width, &height,&baseline);

  x = (winfo.width - width) / 2;
  y = (winfo.height - height) / 2;

  GrText(GR_ROOT_WINDOW_ID, gc, 
	 winfo.x + x, winfo.y + y, cat->title, -1, GR_TFASCII|GR_TFTOP);

  GrDestroyFont(fnt);
  GrDestroyGC(pgc);
  GrDestroyGC(gc);
}

static void do_cat_switch(void *window)
{
  switch_catagory((int) window);
}

static void catswitch_wndproc(win *window, GR_EVENT *ep)
{
  GR_WINDOW_INFO winfo;

  CATAGORY *curcat = get_current_catagory();

  CATAGORY *newcat = 0;

  switch(ep->type)
    {
    case GR_EVENT_TYPE_BUTTON_DOWN:
      GrGetWindowInfo(ep->button.wid, &winfo);

	  /* Changed this so if the pointer is on the right half of the window it
	  ** will fetch the next catagory, and if on the left half of the window it
	  ** will fetch the previous catagory --- j webb (01/19/01)
	  */
		if (ep->button.x <= (winfo.width / 2))
			newcat = get_previous_catagory(curcat);
		else if (ep->button.x > (winfo.width / 2))
			newcat = get_next_catagory(curcat);

      if (newcat)
	switch_catagory(newcat->index);

      break;
    }
}

void cat_switch_free_memory()
{
}

/* This will create a new catagory switcher */

void cat_switch_create(void)
{
  int depth;

  GR_SCREEN_INFO si;

  GrGetScreenInfo(&si);
      
  switch(si.bpp)
    {
    case 1:
    case 2:
    case 4:
    case 8:
      depth = 1;
      break;

    case 16:
      depth = 2;
      break;

    case 24:
    case 32:
      depth = 4;
      break;
    }

  /* Ok, make a new input window that will look for a button click */
  /* in the catagory switcher */

  cswid = GrNewInputWindow(GR_ROOT_WINDOW_ID, 
			 (si.ws_width - 80) / 2, 0,
			 80, 20);

  /* THis is the offscreen pixmap for the alpha blending */

  cat_pixmap = GrNewPixmap(80, 20, NULL);

  add_window(cswid, GR_ROOT_WINDOW_ID, 0, catswitch_wndproc);
  
  GrSelectEvents(cswid, GR_EVENT_MASK_BUTTON_DOWN | GR_EVENT_MASK_BUTTON_UP);

  GrMapWindow(cswid);  
}

#endif


