#ifndef NXCOMM_H
#define NXCOMM_H

/*
 * NXCOMM.H
 * Copyright 2000, Century Software Embedded Technologies
 */

/* 
 * DESCRIPTION:  This file defines all of the external structures
 *               and functions for the nxcomm library
: *
 * AUTHOR:       First draft, Jordan Crouse, 11/27/00
 */

/* STRUCTURES */
typedef struct
{
	char *addr;
	char *netmask;
	char *broadcast;
	char *subnet;
	char *gateway;
	int  dhcp;
} nxcomm_ipaddr_str_t;

typedef struct
{
	const char *domain;
	const char *search;
	unsigned long long_dns_1;
	unsigned long long_dns_2;
	unsigned long long_dns_3;
	char *str_dns_1;
	char *str_dns_2;
	char *str_dns_3;
} nxcomm_dns_t;


typedef struct
{
  unsigned long addr;
  unsigned long netmask;
  unsigned long broadcast;
} nxcomm_ipaddr_t;

typedef struct
{
  char device[128];
  int speed;
  char telephone[12];
  char account[20];
  char password[20];
  unsigned long local_ipaddr;
  unsigned long remote_ipaddr;
  unsigned long netmask;
  unsigned long flags;
} nxcomm_ppp_options_t;

typedef struct
{
  char name[15];
  unsigned char type;
  unsigned char active;
} nxcomm_interface_t;

typedef struct
{
  unsigned long in_bytes;
  unsigned long out_bytes;
} nxcomm_ppp_stats_t;

typedef struct
{
  char device[128];
  char ifname[25];
  char remote_ipaddr[20];
  char local_ipaddr[20];
  int  speed;
  unsigned long start_time;
} nxcomm_ppp_info_t;

typedef struct
{
  char name[16];
  unsigned short status;
  unsigned char quality;
  unsigned char level;
  unsigned char noise;
  struct
  {
    unsigned long nwid;
    unsigned long code;
    unsigned long misc;
  } discard;
} nxcomm_wirestats_t;

/* Array Sizes */
#define NXCOMM_IP_LEN			 16

/* PPP Types */
#define NXCOMM_PPP_NULLMODEM   1
#define NXCOMM_PPP_MODEM       2

#define NXCOMM_PPP_AUTH        0x01
#define NXCOMM_PPP_HWFLOW      0x02

#define NXCOMM_TYPE_PPP        0x01
#define NXCOMM_TYPE_ETHERNET   0x02
#define NXCOMM_TYPE_ANY        0x03

/* ERROR CODES */
#define NXCOMM_OK              0
#define NXCOMM_ERROR           -1
#define NXCOMM_NOT_IMPLEMENTED -2
#define NXCOMM_IPADDR_ERROR    -3
#define NXCOMM_NO_INTERFACES   -4
#define NXCOMM_ACTIVE          -5
#define NXCOMM_INACTIVE        -6
#define NXCOMM_INVALID         -7

/* Internal codes, not normally used by the outside world */
/* These are used for a IOCTL type of interaction between the API and */
/* processor specific functions */

#define NXCOMM_GET 1
#define NXCOMM_SET 2
#define NXCOMM_ADD 3
#define NXCOMM_DEL 4
    
int nxcomm_set_ip_address(char *ifr, nxcomm_ipaddr_t *);
int nxcomm_get_ip_address(char *ifr, nxcomm_ipaddr_t *);
int nxcomm_add_default_gateway(unsigned long addr, char *interface);
int nxcomm_remove_default_gateway();
int nxcomm_get_default_gateway(unsigned long *addr);

int nxcomm_if_up(char *ifname, int dhcp);
int nxcomm_if_down(char *ifname);

int nxcomm_set_nameserver(char *, char *, unsigned long *, int);

int nxcomm_get_if_list(int, nxcomm_interface_t *, int *);

int nxcomm_wireless_set_essid(char *interface, char *name);
int nxcomm_wireless_get_essid(char *interface, char *name);

int nxcomm_wireless_get_stats(char *interface, nxcomm_wirestats_t *stats);
int nxcomm_wireless_get_if_list(nxcomm_interface_t *, int *);

int nxcomm_ppp_connect(int, nxcomm_ppp_options_t *, char **, int, int, char *);

int nxcomm_ppp_disconnect(char *);
int nxcomm_if_active(char *);
int nxcomm_ppp_get_stats(char *, nxcomm_ppp_stats_t *);
int nxcomm_ppp_get_ip_info(char *, nxcomm_ppp_options_t *);

int nxcomm_ppp_write_info(char *filename, nxcomm_ppp_info_t *pppinfo);
int nxcomm_ppp_read_info(char *filename, nxcomm_ppp_info_t *pppinfo);

/* These are macros that abstract the ppp_connect call */
#define nxcomm_dialup_ppp_connect(opts, cmds, count) \
                                 (nxcomm_ppp_connect(NXCOMM_PPP_MODEM, opts, cmds, count))

#define nxcomm_direct_ppp_connect(opts) \
                                 (nxcomm_ppp_connect(NXCOMM_PPP_NULLMODEM, opts, 0, 0))

#endif
