#ifndef NXIO_H
#define NXIO_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct
{
  unsigned char left;
  unsigned char right;
} nxio_level_t;

typedef struct 
{
  unsigned char stereo;
  unsigned char sample;
  int speed;
  unsigned long size;
  int blocksize;
} nxio_audio_t;

  
#define NXIO_MONO 0
#define NXIO_STEREO 1

#define NXIO_DEFAULT_RATE 44100

#define NXIO_WAV_FILE 0x01

#define NXIO_PLAY   0x01
#define NXIO_RECORD 0x02

/* Mixer devices */

/* These are equal to the Linux settings */
/* for other architectures, you need a   */
/* conversion macro to your settings     */

#define NXIO_MIXER_COUNT  6

#define NXIO_MIXER_VOLUME 0
#define NXIO_MIXER_BASS   1
#define NXIO_MIXER_TREBLE 2
#define NXIO_MIXER_MIC    3
#define NXIO_MIXER_LINE1  4
#define NXIO_MIXER_LINE2  5

#define NXIO_MIXER_LABELS {"Vol ", "Bass ", "Trebl", \
                           "Mic ", "Line1", "Line2"}

/* Error Messages */

#define NXIO_OK               0
#define NXIO_ERROR            -1
#define NXIO_NOT_IMPLEMENTED  -2
#define NXIO_BAD_DEVICE       -3
#define NXIO_AGAIN            -4 
#define NXIO_FILE_TOO_BIG     -5

int nxio_play_soundfile(int type, char *filename);

int nxio_save_sound_file(int type, char *filename, 
			  nxio_audio_t *settings, unsigned char *buffer);

int nxio_load_sound_file(int type, char *filename, 
			  nxio_audio_t *settings, unsigned char *buffer, int size);

int nxio_get_wav_stats(char *filename, nxio_audio_t *settings);

int nxio_open_stream(int direction, nxio_audio_t *settings);
int nxio_stream_record(int fd, unsigned char *buffer, int size);
int nxio_stream_play(int fd, unsigned char *buffer, int size);

int nxio_get_mixer_devices(unsigned long *bitmask);

int nxio_set_mixer_level(int device, nxio_level_t *level);
int nxio_get_mixer_level(int device, nxio_level_t *level);

#define nxio_set_volume(level) \
        nxio_set_mixer_level(NXIO_MIXER_VOLUME, level)

#define nxio_get_volume(level) \
        nxio_get_mixer_level(NXIO_MIXER_VOLUME, level)


#ifdef __cplusplus
}
#endif

#endif
