#ifndef NXSYS_H
#define NXSYS_H

/*
 * NXSYS.H
 * Copyright 2000, Century Software Embedded Technologies
 */

/* 
 * DESCRIPTION:  This file defines all of the external structures
 *               and functions for the nxsys library
 *
 * AUTHOR:       First draft, Jordan Crouse, 11/27/00
 */

/* STRUCTURES */

#include "timelib.h"
typedef struct
{
	char *addr;
	char *netmask;
	char *broadcast;
	char *subnet;
	char *gateway;
	int  dhcp;
} nxsys_ipaddr_str_t;

typedef struct
{
	const char *domain;
	const char *search;
	unsigned long long_dns_1;
	unsigned long long_dns_2;
	unsigned long long_dns_3;
	char *str_dns_1;
	char *str_dns_2;
	char *str_dns_3;
} nxsys_dns_t;

typedef struct
{
  unsigned long addr;
  unsigned long netmask;
  unsigned long broadcast;
} nxsys_ipaddr_t;

typedef struct
{
  char device[128];
  int speed;
  char telephone[12];
  char account[20];
  char password[20];
  unsigned long local_ipaddr;
  unsigned long remote_ipaddr;
  unsigned long netmask;
} nxsys_ppp_options_t;

/* Note:  The following two structures are pretty        */
/* *NIX specific.  The reason being that most apps       */
/* will want to know all of the available information    */
/* If your platform isn't *NIX based, you can either     */
/* pass dummy values for those unused fields, or use     */
/* a IFDEF to define an alternate struct for your system */

typedef struct
{
  unsigned long user;   /* User CPU time */
  unsigned long nice;   /* CPU time of "nice" prodcesses */
  unsigned long system; /* Total kernel CPU time */
  unsigned long idle;   /* Total idle time */
} nxsys_cpu_t;

typedef struct
{
  unsigned long total;   /* Total RAM */
  unsigned long used;    /* Total used RAM */
  unsigned long free;    /* Free RAM */
  unsigned long shared;  /* shared RAM (user & kernel) */
  unsigned long buffers; /* RAM allocated to kernel buffers */
  unsigned long cached;  /* cached RAM */
  unsigned long stotal;  /* Total swap space */
  unsigned long sused;   /* Total swap used */
} nxsys_memory_t;

typedef struct
{
  unsigned char flags;
  unsigned char percent;
  unsigned short battery_life;
} nxsys_battery_t;

/* Backlight power control */
#define NXSYS_BLITE_POWER_ON  1
#define NXSYS_BLITE_POWER_OFF 0

/* PPP Flags */
#define NXSYS_PPP_NULLMODEM   1
#define NXSYS_PPP_MODEM       2

/* Battery info flags */

#define NXSYS_BATTERY_AC_OFFLINE 0x0
#define NXSYS_BATTERY_AC_ONLINE  0x1  
#define NXSYS_BATTERY_ACTIVE     0x2 
#define NXSYS_BATTERY_CHARGING   0x4

#define NXSYS_BATTERY_PRESENT    0x6  /* Active and/or charging */

/* ERROR CODES */
#define NXSYS_OK              0
#define NXSYS_ERROR           1
#define NXSYS_NOT_IMPLEMENTED 2
#define NXSYS_BAD_VALUE       3
#define NXSYS_NOT_SUPERUSER   4
#define NXSYS_IPADDR_ERROR    5
#define NXSYS_FILE_ERROR      6

/* FUNCTION PROTOTYPES */

/* SYSTEM SETTINGS */
#ifdef __cplusplus
extern "C" {
#endif
int nxsys_set_date(nxsys_date_t *);
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
extern "C" {
#endif
int nxsys_get_date(nxsys_date_t *);
#ifdef __cplusplus
}
#endif

/* NETWORKING SETTINGS */
#ifdef __cplusplus
extern "C" {
#endif
int nxsys_set_ip_address(char *ifr, nxsys_ipaddr_t *);
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
extern "C" {
int nxsys_get_ip_address(char *ifr, nxsys_ipaddr_t *);
#endif
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
extern "C" {
#endif
int nxsys_get_net_value(char *,char *);
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
extern "C" {
#endif 
int nxsys_write_net_values(nxsys_ipaddr_str_t, nxsys_dns_t);
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
extern "C" {
#endif
int nxsys_add_default_gateway(unsigned long addr);
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
extern "C" {
#endif
int nxsys_remove_default_gateway();
#ifdef __cplusplus
}
#endif


#ifdef __cplusplus
extern "C" {
#endif
int nxsys_ppp_connect(int, nxsys_ppp_options_t *, char **, int);
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
extern "C" {
#endif
int nxsys_ppp_disconnect();
#ifdef __cplusplus
}
#endif


/* POWER MANAGEMENT */
#ifdef __cplusplus
extern "C" {
#endif
int nxsys_set_backlight(unsigned char, unsigned char);
#ifdef __cplusplus
}
#endif


/* STATISTICS */
#ifdef __cplusplus
extern "C" {
#endif
int nxsys_get_cpu_load(nxsys_cpu_t *);
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
extern "C" {
#endif
int nxsys_get_memory_usage(nxsys_memory_t *);
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
extern "C" {
#endif
int nxsys_get_battery(nxsys_battery_t *);
#ifdef __cplusplus
}
#endif


#endif
