/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: TIMELIB.H                                                  *
*                                                                      *
***********************************************************************/
#ifndef TIMELIB_H
#define TIMELIB_H



/*
**
** Imported "Include" Files
**
*/
#include <time.h>



/*
**
** Global Structure Definitions
**
*/
typedef struct /* 7 bytes */
{
  unsigned short year;
  unsigned char month;
  unsigned char day;
  unsigned char hour;
  unsigned char minute;
  unsigned char second;
}nxsys_date_t;



/*
**
**  NAME: nxsysConvertTMStruct()
**
** USAGE: int nxsysConvertTMStruct(struct tm *tstruct, nxsys_date_t *result);
**
** DESCR: This function will convert the time/date information found in the
**        specified 'tm' structure into an nxsys_date_t structure.
**
** PARMS: The "tstruct" parameter is a pointer to the 'tm' structure. The
**        "result" parameter is a pointer to an nxsys_date_t structure
**        that will contain the result.
**
** RETRN: If successful, '0' is returned. If the time/date specified by the
**        'tm' structure is outside the bounds of an nxsys_date_t
**        structure, a non-zero value is returned.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
int nxsysConvertTMStruct(struct tm *, nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysGetCurrentTime()
**
** USAGE: void nxsysGetCurrentTime(nxsys_date_t *result);
**
** DESCR: This function will initialize an nxsys_date_t structure with
**        the current time and date.
**
** PARMS: The "result" parameter points to the nxsys_date_t structure
**        that will contain the result.
**
** RETRN: Nothing.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
void nxsysGetCurrentTime(nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysCompareDates()
**
** USAGE: int nxsysCompareDates(nxsys_date_t *stamp1, nxsys_date_t *stamp2);
**
** DESCR: This function will compare the dates found in two nxsys_date_t
**        structures.
**
** PARMS: The "stamp1" and "stamp2" parameters are pointers to initialized
**        nxsys_date_t structures.
**
** RETRN: If the first date is earlier than the second, a negative
**        value is returned. If the first date is later than the second,
**        a positive value is returned. If the two dates are the same,
**        '0' is returned.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
int nxsysCompareDates(nxsys_date_t *, nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysCompareTimeStamps()
**
** USAGE: int nxsysCompareTimeStamps(nxsys_date_t *stamp1, nxsys_date_t *stamp2);
**
** DESCR: The following function will compare the dates and times found
**        in two nxsys_date_t structures.
**
** PARMS: The "stamp1" and "stamp2" parameters are pointers to initialized
**        nxsys_date_t structures.
**
** RETRN: If the first timestamp is earlier than the second, a negative
**        value is returned. If the first timestamp is later than the
**        second, a positive value is returned. If the two timestamps
**        are the same, '0' is returned.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
int nxsysCompareTimeStamps(nxsys_date_t *, nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysCompareBounds()
**
** USAGE: int nxsysCompareBounds(nxsys_date_t *first,
**                               nxsys_date_t *last,
**                               nxsys_date_t *subject);
**
** DESCR: This function will check to see if the subject timestamp falls
**        within the bounds specified by the first and last timestamps.
**
** PARMS: The "first" and "last" parameters are pointers to initialized
**        nxsys_date_t structures that define the bounds. The "subject"
**        parameter is a pointer to an initialized nxsys_date_t structure
**        that specifies the time to be tested.
**
** RETRN: If the subject timestamp is greater than or equal to the first
**        timestamp and is less than or equal to the last timestamp, '0'
**        is returned. Otherwise, the subject timestamp is outside of the
**        bounds specified and a non-zero value is returned.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
int nxsysCompareBounds(nxsys_date_t *, nxsys_date_t *, nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysIncrementDate()
**
** USAGE: void nxsysIncrementDate(nxsys_date_t *stamp);
**
** DESCR: This function will add one day to the date found in an
**        nxsys_date_t structure.
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure.
**
** RETRN: Nothing.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
void nxsysIncrementDate(nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysDecrementDate()
**
** USAGE: void nxsysDecrementDate(nxsys_date_t *stamp);
**
** DESCR: This function will subtract one day from the date found in an
**        nxsys_date_t structure.
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure.
**
** RETRN: Nothing.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
void nxsysDecrementDate(nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysIncrementHour()
**
** USAGE: void nxsysIncrementHour(nxsys_date_t *stamp);
**
** DESCR: This funtion will add one hour to the date found in an nxsys_date_t
**        structure.
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure.
**
** RETRN: Nothing.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
void nxsysIncrementHour(nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysDecrementHour()
**
** USAGE: void nxsysDecrementHour(nxsys_date_t *stamp);
**
** DESCR: This funtion will subtract one hour from the date found in an
**        nxsys_date_t structure.
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure.
**
** RETRN: Nothing.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
void nxsysDecrementHour(nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysIncrementMinute()
**
** USAGE: void nxsysIncrementMinute(nxsys_date_t *stamp);
**
** DESCR: This funtion will add one minute to the date found in an nxsys_date_t
**        structure.
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure.
**
** RETRN: Nothing.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
void nxsysIncrementMinute(nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysDecrementMinute()
**
** USAGE: void nxsysDecrementMinute(nxsys_date_t *stamp);
**
** DESCR: This funtion will subtract one minute from the date found in an
**        nxsys_date_t structure.
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure.
**
** RETRN: Nothing.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
void nxsysDecrementMinute(nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysIncrementSecond()
**
** USAGE: void nxsysIncrementSecond(nxsys_date_t *stamp);
**
** DESCR: This funtion will add one second to the date found in an nxsys_date_t
**        structure.
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure.
**
** RETRN: Nothing.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
void nxsysIncrementSecond(nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysDecrementSecond()
**
** USAGE: void nxsysDecrementSecond(nxsys_date_t *stamp);
**
** DESCR: This funtion will subtract one second from the date found in an
**        nxsys_date_t structure.
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure.
**
** RETRN: Nothing.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
void nxsysDecrementSecond(nxsys_date_t *);
#ifdef __cplusplus
}
#endif




/*
**
**  NAME: nxsysGetFullDayOfWeek()
**
** USAGE: char *nxsysGetDayOfWeek(nxsys_date_t *stamp);
**
** DESCR: This function will calculate what day of the week corresponds
**        to the specified nxsys_date_t structure.
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure.
**
** RETRN: A pointer to the text string that specifies the day of the week
**        is returned.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
char *nxsysGetFullDayOfWeek(nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysGetShortDayOfWeek()
**
** USAGE: char *nxsysGetDayOfWeek(nxsys_date_t *stamp);
**
** DESCR: This function will calculate what day of the week corresponds
**        to the specified nxsys_date_t structure.
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure.
**
** RETRN: A pointer to the text string that specifies the day of the week
**        is returned.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
char *nxsysGetShortDayOfWeek(nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysGetShortMonth()
**
** USAGE: char *nxsysGetShortMonth(unsigned char month);
**
** DESCR: This function will recover a pointer to a text string that describes
**        the specified month with a three letter abbreviation.
**
** PARMS: The "month" parameter specifies the month's numeric value.
**
** RETRN: If successful, a pointer to the text string for the specified month
**        is returned. If an error occures during function execution, a pointer
**        to an empty string is returned.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
char *nxsysGetShortMonth(unsigned char);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysGetFullMonth()
**
** USAGE: char *nxsysGetFullMonth(unsigned char month);
**
** DESCR: This function will recover a pointer to a text string that describes
**        the specified month.
**
** PARMS: The "month" parameter specifies the month's numeric value.
**
** RETRN: If successful, a pointer to the text string for the specified month
**        is returned. If an error occures during function execution, a pointer
**        to an empty string is returned.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
char *nxsysGetFullMonth(unsigned char);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysGetDaysInMonth()
**
** USAGE: int nxsysGetDaysInMonth(nxsys_date_t *stamp);
**
** DESCR: This function will recover the number of days in the month specified
**        in the associated nxsys_date_t structure. Leap year calculations are
**        included.
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure.
**
** RETRN: If successful, the number of days in the specified month are returned.
**        If an error occures during function execution, '0' is returned.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
int nxsysGetDaysInMonth(nxsys_date_t *);
#ifdef __cplusplus
}
#endif



/*
**
**  NAME: nxsysGetTimeDateString()
**
** USAGE: void nxsysGetTimeDateString(nxsys_date_t *stamp,
**                                    short format,
**                                    char *result);
**
** DESCR: This function will create an ASCIIZ string that represents the date
**        and time found in the specified nxsys_date_t structure. The resulting
**        string will be in one of the following formats:
**
**        format=0:   June 27, 2001
**        format=1:   June 27, 2001, 10:58 AM
**        format=2:   June 27, 2001, 10:58:30 AM
**        format=3:   Wednesday, June 27, 2001
**        format=4:   Wednesday, June 27, 2001, 10:58 AM
**        format=5:   Wednesday, June 27, 2001, 10:58:30 AM
**        format=6:   06/27/01
**        format=7:   06/27/01 10:58a
**        format=8:   06/27/01 10:58:30a
**        format=9:   Wed 06/27/01
**        format=10:  Wed 06/27/01 10:58a
**        format=11:  Wed 06/27/01 10:58:30a
**        format=12:  27 Jun 2001
**        format=13:  27 Jun 2001 10:58 AM
**        format=14:  27 Jun 2001 10:58:00 AM
**        format=15:  Wed 27 Jun 2001
**        format=16:  Wed 27 Jun 2001 10:58 AM
**        format=17:  Wed 27 Jun 2001 10:58:30 AM
**        format=18:  10:58 AM
**        format=19:  10:58:30 AM
**
** PARMS: The "stamp" parameter is a pointer to an initialized nxsys_date_t
**        structure. The "format" parameter specifies the string's format.
**        The "result" parameter is a pointer to the buffer where the 
**        resulting ASCIIZ string will be stored.
**
** RETRN: Nothing.
**
*/
#ifdef __cplusplus
extern "C" {
#endif
void nxsysGetTimeDateString(nxsys_date_t *, short, char *);
#ifdef __cplusplus
}
#endif

#endif /* TIMELIB_H */
