/*
 * Nano-X Directory Search routine
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include "nano-X.h"
#include "nxlib.h"

/* search passed directory for files according to flag options*/
int
nxSearchDir(char *dir, char *pattern, int flags, nxEnumProc cbProc, void *data)
{
  int outmode = 0;

	DIR *		fp;
	struct dirent *	pdirent;
	int		count = 0;
	struct stat 	sbuf;
	char 		path[256];

	fp = opendir(dir);
	if (fp) 
	  {
	    while((pdirent = readdir(fp)) != NULL) 
	      {
		sprintf(path, "%s/%s", dir, pdirent->d_name);
		if (stat(path, &sbuf) == 0) 
		  {
		    outmode = 0;

		    /* Filter out those file we don't want */
		    if ((flags & NXDIR_FILE) && !(S_ISREG(sbuf.st_mode)))
		      continue;

		    if ((flags & NXDIR_EXEC) && !(sbuf.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH)))
		      continue;

		    if (S_ISREG(sbuf.st_mode))
		      outmode |= NXDIR_FILE;
		    
		    if (sbuf.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH))
		      outmode |= NXDIR_EXEC;

		    ++count;
		    if (!cbProc(path, outmode, data))
		      break;
		  }
	      }
	    closedir(fp);
	  }
	return count;
}

/* Search for the filename in the given searchdirs and return the
   full path in fullpath */

int nxSearchForFile(char *filename, char *fullpath, int pathsize, 
		    char **searchdirs, int searchcount)
{
  int i;
  char searchpath[512];

  /* Go through each of the searchdirs and look for our file */
  
  for(i = 0; i < searchcount; i++)
    {
      sprintf(searchpath, "%s/%s", searchdirs[i], filename);

      if (!access(searchpath, R_OK))
	{
	  strzcpy(fullpath, searchpath, pathsize);
	  return(1);
	}
    }
  
  return(0);
}
