/* 
 * LINUX_WIRELESS.C 
 * Routines for managing 802.11 wireless connections
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#include <sys/socket.h>
#include <sys/ioctl.h>

#include <linux/wireless.h>

#include "nxcomm.h"
#include "linux_wireless.h"

/* Defined in linux_if.c */

int linux_open_socket();

static int wireless_do_ioctl(char *interface, int cmd, struct iwreq *data)
{
  int ret;
  int fd = linux_open_socket();

  if (fd < 0) return(errno);
  
  strcpy(data->ifr_ifrn.ifrn_name, interface);
  
  /* Now, do the ioctl call */
  ret = ioctl(fd, cmd, data);

  if (ret < 0)
    ret = errno;
  
  close(fd);

  return(ret);
}

int wireless_essid(int operation, char *interface, char *name)
{
  int cmd = 0;
  struct iwreq data;
  
  switch(operation)
    {
    case NXCOMM_GET:
      cmd = SIOCGIWESSID;
      break;

    case NXCOMM_SET:
      cmd = SIOCSIWESSID;
      break;
    }

  data.u.essid.pointer = (caddr_t) name;

  if (wireless_do_ioctl(interface, cmd, &data) < 0)
    return(NXCOMM_ERROR);
  else
    return(NXCOMM_OK);
}

int wireless_get_statistics(char *interface, nxcomm_wirestats_t *stats)
{
  char tmpstr[3];
  char buffer[BUFSIZ];

  FILE* wfile = fopen("/proc/net/wireless", "r");
  
  if (NULL == wfile) return(NXCOMM_ERROR);
  
  /* Read the first couple of lines */
  fgets(buffer, BUFSIZ, wfile);
  fgets(buffer, BUFSIZ, wfile);

  /* Now, go through all the interfaces until we find one that */
  /* we want */

  while(1)
    {
      if (feof(wfile)) break;
      
      fgets(buffer, BUFSIZ, wfile);
      if (!strncmp(buffer, interface, strlen(interface)))
	{	  
	  sscanf(buffer, "%6s: %04hx  %3hhd%c  %3hhd%c  %3hhd%c  %6ld %6ld %6ld\n",
		 stats->name,
		 &(stats->status),
		 &(stats->quality),
		 &tmpstr[0],
		 &stats->level,
		 &tmpstr[1],
		 &stats->noise,
		 &tmpstr[2],
		 &stats->discard.nwid,
		 &stats->discard.code,
		 &stats->discard.misc);

	  fclose(wfile);
	  return(NXCOMM_OK);
	}
    }

  fclose(wfile);
  return(NXCOMM_ERROR);
}

int wireless_get_if_list(nxcomm_interface_t *iflist, int *ifcount) 
{
  char tmpstr[3];
	char ifname[10];
  char buffer[BUFSIZ];
	int idx = 0;
  int retcount = -1;
  int err = NXCOMM_ERROR;
	nxcomm_wirestats_t stats;
	
  /* We want to return a list of interfaces and some stats */
  FILE* wfile = fopen("/proc/net/wireless", "r");
  
  if (NULL == wfile) 
		return(NXCOMM_ERROR);

  	/* Read the first couple of lines */
  	fgets(buffer, BUFSIZ, wfile);
  	fgets(buffer, BUFSIZ, wfile);
	
	while (1)
	{
		if (feof(wfile)) 
			break;
		fgets(buffer, BUFSIZ, wfile);	
	  	sscanf(buffer, "%6s: %04hx  %3hhd%c  %3hhd%c  %3hhd%c  %6ld %6ld %6ld\n",
			stats.name,
			&(stats.status),
			&(stats.quality),
		 	&tmpstr[0],
		 	&stats.level,
		 	&tmpstr[1],
		 	&stats.noise,
		 	&tmpstr[2],
		 	&stats.discard.nwid,
		 	&stats.discard.code,
		 	&stats.discard.misc);
		
		for (idx = 0; idx < 4; idx++) 
		{
			sprintf(ifname,"eth%d",idx);			
			if (0 == strncmp(ifname, stats.name, strlen(ifname)))
			{		 
  				retcount++;
	  			strcpy(iflist[retcount].name, ifname);
				if (retcount >= *ifcount)
						break;
			}
			sprintf(ifname,"wvlan%d",idx);
			if (0 == strncmp(ifname, stats.name, strlen(ifname)))
			{
				retcount++;
				strcpy(iflist[retcount].name, ifname);
				if (retcount >= *ifcount)
					break;
			}
		}
	} 
					 
	if (retcount != -1)
  	{
   	err = NXCOMM_OK;
  		*ifcount = retcount;
  	}
  	else
	{
  		err = NXCOMM_NO_INTERFACES;
  		*ifcount = 0;
	}
	fclose(wfile);
	return (err);
}
	  
  
