/*
 * NXCOMM_API.C
 * Copyright 2000, Century Software Embedded Technologies
 */

/* 
 * DESCRIPTION:  This is the official external API for
 *               libnxcomm, the networking interation layer for
 *               Microwindows and Nano-X.  All functions
 *               within are intended to be used by client 
 *               aplications 
 *
 * AUTHOR:       First draft, Jordan Crouse, 11/27/00
 */

#include "nxcomm.h"
#include "nxcomm_proto.h"

/* This is a structure (defined elsewhere for each 
 * individual platform), that give us a gateway to the
 * actualy system dependent functions for each API 
 * call 
 */

extern nxcomm_func_t nxcomm_functions;

/*
 * nxcomm_set_ip_address()
 * This function sets the IP settings for the specified interface 
 */

int nxcomm_set_ip_address(char *ifr, nxcomm_ipaddr_t *ip_addr)
{
  if (nxcomm_functions.sysdep_do_ip_address)
    return(nxcomm_functions.sysdep_do_ip_address(NXCOMM_SET, ifr, ip_addr));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_get_ip_address(char *ifr, nxcomm_ipaddr_t *ip_addr)
{
  if (nxcomm_functions.sysdep_do_ip_address)
    return(nxcomm_functions.sysdep_do_ip_address(NXCOMM_GET, ifr, ip_addr));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

/*
 * nxcomm_set_nameserver()
 * This function sets the nameserver address and domain
 */

int nxcomm_set_nameserver(char *domain, char *search, unsigned long *addr, int acount)
{
  if (nxcomm_functions.sysdep_set_nameserver)
    return(nxcomm_functions.sysdep_set_nameserver(domain, search, addr, acount));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

/*
 * nxcomm_add_default_gateway()
 * This function adds a IP default gateway to the system
 */

int nxcomm_add_default_gateway(unsigned long addr, char *interface)
{
  if (nxcomm_functions.sysdep_do_default_gateway)
    return(nxcomm_functions.sysdep_do_default_gateway(NXCOMM_ADD, &addr, interface));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_remove_default_gateway()
{
  if (nxcomm_functions.sysdep_do_default_gateway)
    return(nxcomm_functions.sysdep_do_default_gateway(NXCOMM_DEL, 0, 0));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_get_default_gateway(unsigned long *addr)
{
  if (nxcomm_functions.sysdep_do_default_gateway)
    return(nxcomm_functions.sysdep_do_default_gateway(NXCOMM_GET, addr, 0));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}


int nxcomm_ppp_connect(int ctype, nxcomm_ppp_options_t *opts, char **cmds, 
				int cmdcount, int dhcp, char *config)
{
  if (nxcomm_functions.sysdep_ppp_connect)
    return(nxcomm_functions.sysdep_ppp_connect(ctype, opts, cmds, cmdcount, dhcp, config));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_ppp_disconnect(char *ifname)
{
  if (nxcomm_functions.sysdep_ppp_disconnect)
    return(nxcomm_functions.sysdep_ppp_disconnect(ifname));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_get_if_list(int filter, nxcomm_interface_t *iflist, int *ifcount)
{
  if (nxcomm_functions.sysdep_get_if_list)
    return(nxcomm_functions.sysdep_get_if_list(filter, iflist, ifcount));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_if_active(char *ifname)
{
  if (nxcomm_functions.sysdep_if_active)
    return(nxcomm_functions.sysdep_if_active(ifname));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_if_up(char *ifname, int dhcp)
{
  if (0 == dhcp)
    {
      if (nxcomm_functions.sysdep_set_if_status)
        return(nxcomm_functions.sysdep_set_if_status(NXCOMM_ACTIVE, ifname));
      else
        return(NXCOMM_NOT_IMPLEMENTED);
	 }
  else
	 {
		if (nxcomm_functions.sysdep_if_dhcp)
		  return(nxcomm_functions.sysdep_if_dhcp(ifname));
		else
			return(NXCOMM_NOT_IMPLEMENTED);
	 }
}

int nxcomm_if_down(char *ifname)
{
  if (nxcomm_functions.sysdep_set_if_status)
    return(nxcomm_functions.sysdep_set_if_status(NXCOMM_INACTIVE, ifname));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_ppp_get_stats(char *ifname, nxcomm_ppp_stats_t *pppstats)
{
  if (nxcomm_functions.sysdep_ppp_get_stats)
    return(nxcomm_functions.sysdep_ppp_get_stats(ifname, pppstats));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_ppp_write_info(char *filename, nxcomm_ppp_info_t *pppinfo)
{
  if (nxcomm_functions.sysdep_ppp_write_info)
    return(nxcomm_functions.sysdep_ppp_write_info(filename, pppinfo));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_ppp_read_info(char *filename, nxcomm_ppp_info_t *pppinfo)
{
  if (nxcomm_functions.sysdep_ppp_read_info)
    return(nxcomm_functions.sysdep_ppp_read_info(filename, pppinfo));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_ppp_get_ip_info(char *ifname, nxcomm_ppp_options_t *pppoptions)
{
	if (nxcomm_functions.sysdep_ppp_get_ip_info)
		return(nxcomm_functions.sysdep_ppp_get_ip_info(ifname, pppoptions));
	else
		return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_wireless_set_essid(char *interface, char *name)
{
  if (nxcomm_functions.sysdep_wireless_essid)
    return(nxcomm_functions.sysdep_wireless_essid(NXCOMM_SET, interface, name));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_wireless_get_essid(char *interface, char *name)
{
  if (nxcomm_functions.sysdep_wireless_essid)
    return(nxcomm_functions.sysdep_wireless_essid(NXCOMM_GET, interface, name));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_wireless_get_stats(char *interface, nxcomm_wirestats_t *stats)
{
  if (nxcomm_functions.sysdep_wireless_get_stats)
    return(nxcomm_functions.sysdep_wireless_get_stats(interface, stats));
  else
    return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_wireless_get_if_list(nxcomm_interface_t *iflist, int *ifcount)
{
	if (nxcomm_functions.sysdep_wireless_get_if_list)
	 	return(nxcomm_functions.sysdep_wireless_get_if_list(iflist, ifcount));
	else
	 	return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_get_net_value(char *key, char *ret)
{	
  if (nxcomm_functions.sysdep_get_net_value)
	 	return(nxcomm_functions.sysdep_get_net_value(key, ret));
  else
	 	return(NXCOMM_NOT_IMPLEMENTED);
}

int nxcomm_write_net_values(nxcomm_ipaddr_str_t ip_info, nxcomm_dns_t dns_info)
{  
  if (nxcomm_functions.sysdep_write_net_values)
	 	return(nxcomm_functions.sysdep_write_net_values(ip_info, dns_info));
  else
	 	return(NXCOMM_NOT_IMPLEMENTED);
}
