/*
 * NXSYS_API.C
 * Copyright 2000, Century Software Embedded Technologies
 */

/* 
 * DESCRIPTION:  This is the official external API for
 *               libnxsys, the system interation layer for
 *               Microwindows and Nano-X.  All functions
 *               within are intended to be used by client 
 *               aplications 
 *
 * AUTHOR:       First draft, Jordan Crouse, 11/27/00
 */

#include "nxsys.h"
#include "nxsys_proto.h"

/* This is a structure (defined elsewhere for each 
 * individual platform), that give us a gateway to the
 * actualy system dependent functions for each API 
 * call 
 */

extern nxsys_func_t nxsys_functions;

/******** SYSTEM SETTINGS **********/

/* 
 * nxsys_set_date()
 * This function sets the date / time on the system
 */

int nxsys_set_date(nxsys_date_t *date_struct)
{
  if (nxsys_functions.sysdep_set_date)
    return(nxsys_functions.sysdep_set_date(date_struct));
  else
    return(NXSYS_NOT_IMPLEMENTED);
}

/* 
 * nxsys_get_date()
 * This function get the date / time from the system
 */

int nxsys_get_date(nxsys_date_t *date_struct)
{
  if (nxsys_functions.sysdep_get_date)
    return(nxsys_functions.sysdep_get_date(date_struct));
  else
    return(NXSYS_NOT_IMPLEMENTED);
}

/******** POWER MANAGEMENT **********/

/* 
 * nxsys_set_backlight(brightness)
 * This function sets the backlight on the system
 * where brigtness is a percentage of the brightness
 */

int nxsys_set_backlight(unsigned char brightness, unsigned char power)
{
  if (brightness > 100)
    return(NXSYS_ERROR);

  if (nxsys_functions.sysdep_set_backlight)
    return(nxsys_functions.sysdep_set_backlight(brightness, power));
  else
    return(NXSYS_NOT_IMPLEMENTED);
}

/******** STATISTICS **********/

/* 
 * nxsys_get_cpu_load()
 * This function gets the current CPU load 
 */

int nxsys_get_cpu_load(nxsys_cpu_t *cpu_struct)
{
  if (nxsys_functions.sysdep_get_cpu_load)
    return(nxsys_functions.sysdep_get_cpu_load(cpu_struct));
  else
    return(NXSYS_NOT_IMPLEMENTED);
}

/* 
 * nxsys_get_memory_usage()
 * This function gets the current CPU memory usage 
 */

int nxsys_get_memory_usage(nxsys_memory_t *mem_struct)
{
  if (nxsys_functions.sysdep_get_memory_usage)
    return(nxsys_functions.sysdep_get_memory_usage(mem_struct));
  else
    return(NXSYS_NOT_IMPLEMENTED);
}

/* 
 * nxsys_get_battery()
 * This function gets the current battery life
 */

int nxsys_get_battery(nxsys_battery_t *battery_struct)
{
  if (nxsys_functions.sysdep_get_battery)
    return(nxsys_functions.sysdep_get_battery(battery_struct));
  else
    return(NXSYS_NOT_IMPLEMENTED);
}

/*
 * nxsys_get_net_value
 * This function get the value specified by key
 * 
 */
int nxsys_get_net_value(char *key, char *ret)
{	
  if (nxsys_functions.sysdep_get_net_value)
	 return(nxsys_functions.sysdep_get_net_value(key, ret));
  else
	 return(NXSYS_NOT_IMPLEMENTED);
}

/*
 * nxsys_write_net_values
 * This function writes the values to the correct file
 *
 */
int nxsys_write_net_values(nxsys_ipaddr_str_t ip_info, nxsys_dns_t dns_info)
{  
  if (nxsys_functions.sysdep_write_net_values)
	 return(nxsys_functions.sysdep_write_net_values(ip_info, dns_info));
  else
	 return(NXSYS_NOT_IMPLEMENTED);
}
