/*
 * X86.C
 * Copyright 2000, Century Software Embedded Technologies
 */

/* 
 * DESCRIPTION:  This file contains system specific routines
 *               for the Compaq Ipaq device.
 *               
 * AUTHOR:       First draft, Jordan Crouse, 11/27/00
 */

#include <stdio.h>
#include <string.h>

#include "nxcomm.h"
#include "nxsys.h"
#include "linux_if.h"
#include "x86.h"

int x86_set_nameserver(char *domain, char *search, unsigned long *addr, int acount)
{
  /* On the ipaq, we can only write to the /usr/local domain */
  /* so we have a symbolic link from /etc/resolv.conf to     */
  /* /usr/local/etc/resolv.conf, so thats the file we update here */

  return(linux_set_nameserver(RESOLV_FILE, domain, search, addr, acount));
}

int x86_write_net_values(nxsys_ipaddr_str_t ip_info, nxsys_dns_t dns_info)
{
	FILE *net_file;
	unsigned long dns_addr[2];
	int dns_count = 0;
	int ret = 0;

	net_file = fopen(ETH0_FILE, "w");
	if (NULL == net_file) {
		fprintf(stderr, "Unbale to open file %s\n", ETH0_FILE);
		return (NXSYS_ERROR);
	}
	fputs("DEVICE=\"eth0\"\n", net_file);
	fputs("NETMASK=\"", net_file);
	fputs(ip_info.netmask, net_file);
	fputs("\"\n", net_file);
	fputs("ONBOOT=\"yes\"\n", net_file);
	if (1 == ip_info.dhcp) {
		fputs("IPADDR=\"\"\n", net_file);
		fputs("BOOTPROTO=\"dhcp\"\n", net_file);
	}
	else {
		fputs("IPADDR=\"", net_file);
		fputs(ip_info.addr, net_file);
		fputs("\"\n", net_file);
		fputs("BOOTPROTO=\"static\"\n", net_file);
	}
	fclose(net_file);
	
	if (0 != strcmp("...", dns_info.str_dns_1)) {
 		if (0 != strcmp("", dns_info.str_dns_1)) {
			dns_addr[dns_count] = dns_info.long_dns_1;
			dns_count++;
		}
	}
	if (0 != strcmp("...", dns_info.str_dns_2)) {
		 if (0 != strcmp("", dns_info.str_dns_2)) {
			dns_addr[dns_count] = dns_info.long_dns_2;
			dns_count++;
		 }
	}
	if (0 != strcmp("...", dns_info.str_dns_3)) {
		 if (0 != strcmp("", dns_info.str_dns_3)) {
			dns_addr[dns_count] = dns_info.long_dns_3;
			dns_count++;
		 }
	}
	ret = x86_set_nameserver((char *)dns_info.domain, (char *)dns_info.search, dns_addr, dns_count);

	net_file = fopen(X86_GATEWAY_FILE, "w+");
	if (NULL == net_file) {
		fprintf(stderr, "Unbale to open file %s\n", X86_GATEWAY_FILE);
		return (NXSYS_ERROR);
	}
	if (0 == strcmp("...", ip_info.gateway)) {
		fputs("" , net_file);
	}
	else {
		fputs(ip_info.gateway, net_file);
	}
	fclose(net_file);
	return (NXSYS_OK);		
}

int x86_get_net_value(char *key, char *ret)
{
	FILE *net_file;
	char buf[255];
	char *str_p;
	char ret_buf[255];
	int len = 0;
	int idx = 0;	
	int nameserver = 0;
	int target = 0;
	
	if (0 == strcasecmp(key, "DHCP")) {
		key = "BOOTPROTO";
		net_file = fopen(ETH0_FILE, "r");
		if (NULL == net_file) {
			fprintf(stderr, "Unbale to open file %s\n", ETH0_FILE);
			ret = "";
			return (NXSYS_ERROR);
		}
		goto equal;		
	}
	else {
		net_file = fopen(RESOLV_FILE, "r");
		if (NULL == net_file) {
			fprintf(stderr, "Unbale to open file %s\n", ETH0_FILE);
			ret = "";
			return (NXSYS_ERROR);
		}
		goto space;
	}
	ret = "";

equal: //has an = for delim
	while (NULL != fgets(buf, sizeof(buf) - 1, net_file)) {
		if ((str_p =  strstr(buf, key)) && ('#' != buf[0])) {
			ret_buf[0] = '\0';
			str_p = strstr(buf, "=");
			if (NULL != str_p) {
				str_p++;
				if ('\"' == *str_p) {
					str_p++;
					if ('\"' == *str_p || '\n' == *str_p) {
						fclose(net_file);
						ret = "";
						return (NXSYS_OK);
					}
				}
				len = strlen(str_p);
				for (idx = 0; idx < len; idx++) {
					if (('\"' != *str_p) && ('\n' != *str_p) && (NULL != str_p)) {
						ret_buf[idx] = *str_p;
						str_p++;
					}
					else {
						ret_buf[idx] = '\0';
						break;
					}
				}
			}	
			fclose(net_file);
			strncpy(ret, ret_buf, strlen(ret_buf) + 1);
			if (0 == strcasecmp(key, "BOOTPROTO")) {
				if(0 == strcasecmp(ret, "DHCP")) {
					strncpy(ret, "Y", strlen(ret_buf) + 1);
				}
			}
			break;
		}
	}		

space: // has a " " space for a delim
	while (NULL != fgets(buf, sizeof(buf) - 1, net_file)) {
		if (0 == strcmp(key, "DNS_1")) {
			target = 1;
			key = "nameserver";
		}
		if (0 == strcmp(key, "DNS_2")) {
			target = 2;
			key = "nameserver";
		}
		if (0 == strcmp(key, "DNS_3")) {
			target = 3;
			key = "nameserver";
		}
		if (0 == strncasecmp(key, buf, strlen(key))) {
			if (0 == strcmp(key, "nameserver")) {
				nameserver++;
				if (nameserver != target) {
			 		continue;
				}
			}	
			idx = 1;			
			str_p = strstr(buf, " ");
			if (NULL != str_p) {
				str_p++;
				len = strlen(str_p);
				for (idx = 0; idx < len; idx++) {
					ret_buf[idx] = *str_p;
					str_p++;
				}
			}
			ret_buf[idx-1] = '\0';
			fclose(net_file);
			strncpy(ret, ret_buf, strlen(ret_buf) + 1);
			return (NXSYS_OK);
		}
	}
	return (NXSYS_OK);	
}
  
