#include "http.h"
#include <Fl.H>
#include "html.h"
#include "htmlview.h"

#include <vector>
#include <iterator>

extern HTML_WIDGET *g_WebHTML;
extern HTML_WIDGET *g_HTML;

////////////////////////////////////////////////////////////

static void _CheckData(void * pt)
{
  HTTPConnection * pThis = (HTTPConnection*)pt;
  HTTPConnection::HTTPRequest * p = pThis->GetFinished();
  bool						bWebMode;

  vector<HTTPConnection::HTTPRequest *> unmatchedR;

	// This is a hack that is specific to the WebMedia paradigm.  Since it allows 2 browser windows
	// to co-exist in a single window environment, it must be selective as to which html pages are
	// allowed to be parsed, based on the parenting of the current view.

  while(p) {
  	bWebMode = p->HTML()->getKHTMLWidget()->inWebMode();
	// See if we are on the same widget for display....
	if ( (bWebMode && (p->HTML()->getParentView() == g_WebHTML)) ||
		 (bWebMode && (p->HTML() == g_WebHTML)) ||
		 (!bWebMode && (p->HTML()->getParentView() == g_HTML)) ||
		 (!bWebMode && (p->HTML() == g_HTML)) )
	{
    	if(!p->toss()) {
      		if(p->newPage()) {

				p->HTML()->begin(p->getURL());
				p->HTML()->write((const char*)p->getBuffer());
				p->HTML()->end();
				p->HTML()->parse();
				p->HTML()->redraw();
      		} else {
				p->HTML()->data(p->getURL(), (const char *)p->getBuffer(), 
								p->getSize(), true);
      		}
    	}
    	delete p;
	} // end of if
	else
	{
		// Add it back into the queue ???
//		cerr << "Adding request " << p->getURL() << " to unmatched vector\n";
		unmatchedR.push_back(p);

		//	NOTE:
		// 		p is not deleted at this time since future dereferencing
		// 		(i.e. adding it back to the finished queue will cause all
		// 		sorts of ugliness, but will be deleted when it is eventually
		// 		processed and parsed....jw
	} // end of else
    p = pThis->GetFinished();
  }
  
   if(pThis->ReqSize() || pThis->FinishedSize() || pThis->m_pCurrentRequest)
    Fl::add_timeout(.5, _CheckData, pt);

   // Now add back the unmatched requests to the finished queue...
   for (vector<HTTPConnection::HTTPRequest *>::iterator iter = unmatchedR.begin();
		iter != unmatchedR.end();
		++iter)
   {
//	   cerr << "Adding request " << (*iter)->getURL() << " to finishedQ\n";
	   pThis->AddFinished(*iter);
   } // end of for 
}

void _HaveData(HTTPConnection::HTTPRequest * p)
{
  p->HTML()->data(p->getURL(), 0,0, true);
}

void HTTPConnection::slotClick(KHTMLView * v, const char * d1, int b, const char * p)
{

  if(p && p[0]) {
    KHTMLView * v1 = v->findView((const char *)p);
    if(v1)
      v = v1;
  }

  loadPage(v,d1);

  if(m_pDocCallback)
    m_pDocCallback(d1,0,0);

  m_bDocDone = false;
  StartThread();
}

void _CheckDataWrapper(void *p)
{
	_CheckData(p);
}

bool _InWebMode(HTML_WIDGET *p)
{
	bool				brc = false;								// Default to not in webmode

	if (p)
		brc = p->getKHTMLWidget()->inWebMode();

	return brc;
} // end of _InWebMode()

void HTTPConnection::BeginDataTimer()
{
  Fl::add_timeout(.5, _CheckData, (void*)this);
}

void HTTPConnection::do_size()
{
  //  printf("docSize: x=%d y=%d\n",m_pHTML->docWidth(),
  //	 m_pHTML->docHeight());

  //  if(m_pDocCallback)
  //    m_pDocCallback(0,m_pHTML->docWidth(), m_pHTML->docHeight());
}

////////////////////////////////////////////////////////////

