#include <kapp.h>
#include "html.h"
#include <Fl.H>
#ifdef _NANOX
#include <n_x.h>
#else
#include <x.H>
#endif
#include <fl_draw.H>
#include <Fl_Window.H>
#include <Fl_Button.H>
#include <Fl_Pixmap.H>
#include <Fl_Input.H>
#include <Fl_Pack.H>
#include <Fl_Scroll.H>
#include <Fl_Image.H>
#include <Enumerations.H>
#include "http.h"
#include "vmlapp.h"
#include "history.h"
#include "fltk/qconst.h"
#include "fltk/qlist.h"
#include "pixmaps.h"

void Menu_Callback(Fl_Widget * w, void * data);
void Return_Callback(Fl_Widget * w, void * data);
void g_execfunc(const char *cmd, KHTMLWidget *w);

int g_XOffset, g_YOffset, g_ScreenX, g_ScreenY;

class WMPinkWindow : public Fl_Window
{
public:
  WMPinkWindow() : Fl_Window(0,0,g_ScreenX,g_ScreenY) {
    color(fl_color_cube(255*FL_NUM_RED/256, 0*FL_NUM_GREEN/256,255*FL_NUM_BLUE/256));
    callback(Menu_Callback,0);
  }

  int handle(int e) {
    if(e == FL_PUSH) {
      do_callback(this,(void*)0);
    }
  }

};

//#define WMPinkWindow Fl_Window

// Internal forward/back history list
QList<QString> g_History;
int g_HistoryIndex=-1;

// Initial x/y postion on screen
int g_AppX = 0;
int g_AppY = 0;

char g_StartURL[256];

// Initial app width/height
int g_AppWidth = 640;
int g_AppHeight = 480;

Fl_Input * g_URL;
Fl_Button * g_Logo;


extern Fl_Button *Forward;
extern Fl_Button *Backward;
extern Fl_Button *Stop;
extern Fl_Button *Reload;

static char oldstr[256] = "";
static int hindex = 0;

int g_Vert = 0;
Fl_Input * g_Input;

Fl_Window * g_HTMLWindow;
WMPinkWindow * g_PinkWindow;
VMLAppWindow * g_AppWindow = 0;
VMLAppWindow * g_WebWindow = 0;

void LoadPage(const char * URL, HTML_WIDGET * w=0);
VMLAppWindow * make_window();
void _CheckDataWrapper(void *p);

KApplication g_KApp;

#define HTML_WIDGET KHTMLView
#define xHTML_WIDGET KHTMLWidget

void LoadDoc(const char * URL, int w, int h);

HTML_WIDGET * g_HTML;
HTML_WIDGET * g_WebHTML;
HTTPConnection * g_Conn;

#ifdef _NANOX

// the below is needed to fix some X compilation problems with Nano-X
unsigned long KeyPress;
bool XCheckTypedEvent( Display *, unsigned long , 
		       XEvent * )
{
	return false;
}
#endif

// command line parameter struct
typedef struct
{
  char *parm_flag;    // parameter identifier
  void *parm_result;  // where to place parameter value
  int parm_type;      // 0=number | 1=string
}ParmEntry;

ParmEntry parm_list[] =
{
  "-h",&g_AppHeight,0,
  "-u",g_StartURL,1,
  "-w",&g_AppWidth,0,
  "-x",&g_AppX,0,
  "-y",&g_AppY,0,
  NULL,NULL,0
};

static void __cleanup()
{
  delete g_HTML;
  delete g_WebHTML;
  delete g_PinkWindow;
  delete g_HTMLWindow;
  delete g_WebWindow;
  delete g_Conn;
}

int main(int argc, char ** argv)
{
  int index,loop;
  int _x, _y;
  atexit(__cleanup);

  VMLAppWindow * mw;

  // recover any command-line parameters
  g_StartURL[0] = 0x00;
  strcpy(g_StartURL, "file:///home/jeffw/dvlp/projects/nsc/national/html/index.html");
  index = 1;
  while(index < argc)
  {
    loop = 0;
    while(1)
    {
      if(parm_list[loop].parm_flag == NULL)
	break;
      if(strncmp(parm_list[loop].parm_flag,
		 argv[index],
		 strlen(parm_list[loop].parm_flag)) == 0)
      {
	if(parm_list[loop].parm_type) {
	  strcpy(((char *)parm_list[loop].parm_result),
		 argv[index] + strlen(parm_list[loop].parm_flag));
	}
	else
	  *((int *)parm_list[loop].parm_result) = 
	    atoi(argv[index] + strlen(parm_list[loop].parm_flag ));
	break;
      }
      ++loop;
    }
    ++index;
  }
  
  // fake out the widget - "remove" all parms
  argc = 1;

  g_Conn = new HTTPConnection();
  g_Conn->setDocCallback(LoadDoc);

  mw = make_window();

  g_Conn->setHTML(g_HTML);

  mw->show(argc,argv);
  g_WebWindow->show(argc,argv);

#ifdef _NANOX
  GR_SCREEN_INFO i;
  
  GrGetScreenInfo(&i);

  g_ScreenX = i.cols;
  g_ScreenY = i.rows;

  _x = (int)((float)i.cols * .85);
  _y = (int)((float)i.rows * .9);

  g_AppWidth = ( _x > 640 ? 640 : _x );
  g_AppHeight = ( _y> 480 ? 480 : _y );

  g_XOffset = ((i.cols - _x ) / 2) - 15;
  g_YOffset = (i.rows - _y ) / 2;

  cerr << "X,Y = " << i.cols << " " << i.rows << " " << g_ScreenX << " " << g_ScreenY << " " << g_XOffset << " " << g_YOffset << "\n";

  g_AppWindow->resize(g_XOffset,g_YOffset,g_AppWidth,g_AppHeight);
  g_WebWindow->resize(g_XOffset,g_YOffset,g_AppWidth,g_AppHeight);

  g_PinkWindow->resize(0,0,i.cols,i.rows);

#endif

  g_HTML->setGeometry(SCROLLPAD,TITLEHEIGHT ,WIDTH,HEIGHT);
  g_HTML->Fl_Window::begin();

  g_WebHTML->setGeometry(SCROLLPAD,TITLEHEIGHT+35, WIDTH, HEIGHT-35);
  //  g_WebHTML->Fl_Window::begin();

  g_History.setAutoDelete(1);

  cerr << "Main window is " << g_AppWindow << "(" << fl_xid(g_AppWindow) << ")\n";
  cerr << "HTML Widget is " << g_HTML << "(" << fl_xid(g_HTML) << ")\n";
  cerr << "WebHTML Widget is " << g_WebHTML << "(" << fl_xid(g_WebHTML) << ")\n";

#ifdef _NANOX

  GrReparentWindow(fl_xid(g_HTML),fl_xid(g_AppWindow),SCROLLPAD,TITLEHEIGHT);
  GrReparentWindow(fl_xid(g_WebHTML),fl_xid(g_WebWindow),SCROLLPAD,TITLEHEIGHT+35);

#endif

  g_WebWindow->hide();

  if(g_StartURL[0] != 0x00)
    LoadPage(g_StartURL);

  while(Fl::wait()) {}

  return 0;
}

Fl_Button *Forward=(Fl_Button *)0;

//davet
#define MAX_PROTO_STRLEN 7 // must be set to length of longest protocol
static char *protocols[] = 
{
  "http://",
  "file://",
#ifdef OPTIONAL_URI_PROTOCOLS
  "ftp://",
  "afs://",
  "news:",
  "nntp:",
  "mid:",
  "cid:",
  "mailto:",
  "wais://",
  "prospero://",
  "telnet://"
  "gopher://",
#endif
  NULL
};

void SetInput(const char * URL)
{
  if(g_Input) {
    g_Input->value(URL);
    g_Input->set_changed();
    g_Input->mark(256);
  }
}

void _LoadPage(const char * URL, HTML_WIDGET * w=0)
{
  char buf[1024];
  int pindex;

  // get URL protocol
  strncpy(buf,URL,MAX_PROTO_STRLEN);
  buf[MAX_PROTO_STRLEN] = 0x00;

  // check for protocol in list
  pindex = 0;
  while(protocols[pindex] != NULL)
  {
    if(strncmp(buf,protocols[pindex],strlen(protocols[pindex])) == 0)
      break;
    ++pindex;
  }

  // check for missing protocol
  if(protocols[pindex] == NULL)
    sprintf(buf,"http://%s",URL);
  else
    strcpy(buf,URL);

  SetInput(buf);
  
  if(w)
    g_Conn->loadPage(w,buf);
  else
    g_Conn->loadPage(g_HTML,buf);
}

void AddHistory(const char * URL)
{
  if(g_HistoryIndex != -1 && g_HistoryIndex != g_History.count()-1) {
    while(g_HistoryIndex != g_History.count()-1)
      g_History.removeLast();
  }

  g_History.append(new QString(URL));
  g_HistoryIndex = g_History.count() - 1;
}

void LoadPage(const char * URL, HTML_WIDGET * w)
{
  AddHistory(URL);
  _LoadPage(URL,w);

}

static void _LoadPageDelay(void * _url)
{
  //  LoadPage((const char*)_url,g_WebHTML);
}


void LoadPageDelay(int nDelay, HTML_WIDGET * w, const char * url)
{
  //  Fl::add_timeout((double)nDelay,_LoadPageDelay,(void*)url);
  LoadPage((const char*)url,g_WebHTML);  
}

void LoadDoc(const char * URL, int w, int h) 
{
  if(URL) {
    AddHistory(URL);
    SetInput(URL);
  }
  return;
}

void GoPink()
{
  g_PinkWindow->show();
  g_AppWindow->hide();

  

  cerr << "Going pink\n";
}

void GoWeb(const char * url=0)
{
  g_WebWindow->show();
  g_AppWindow->hide();

#ifdef _NANOX
  GrMoveWindow(fl_xid(g_WebWindow),g_XOffset,g_YOffset);
#endif

  if(url)
    LoadPageDelay(10,g_WebHTML,url);
}

void Reload_Callback(Fl_Widget * w, void * data)
{
  _LoadPage(g_Input->value());
}

void Menu_Callback(Fl_Widget * w, void * data)
{
  // Remove the current idbug
  g_execfunc("/bin/wmcmd,idbug,remove,0", g_HTML->getKHTMLWidget());

  g_PinkWindow->hide();
  g_AppWindow->show();

#ifdef _NANOX
  GrMoveWindow(fl_xid(g_AppWindow),g_XOffset,g_YOffset);
#endif

  g_HTML->getKHTMLWidget()->no_fullscreen();
}

void Return_Callback(Fl_Widget * w, void * data)
{
  g_WebWindow->hide();
  g_AppWindow->show();

#ifdef _NANOX
  GrMoveWindow(fl_xid(g_AppWindow),g_XOffset,g_YOffset);
#endif

  g_HTML->getKHTMLWidget()->no_fullscreen();
  g_execfunc("/bin/wmcmd,current,unmute", g_HTML->getKHTMLWidget());

  // If we stopped something from being processed by switching to webmode, start it back up
  if (g_Conn->FinishedSize())
  	_CheckDataWrapper(g_Conn);
}

void Forward_Callback(Fl_Widget * w, void * data)
{
  if(g_HistoryIndex == -1 || g_HistoryIndex+1 > g_History.count()-1)
    return;

  _LoadPage(*g_History.at(++g_HistoryIndex),g_WebHTML);
}

void Backward_Callback(Fl_Widget * w, void * data)
{
  if(g_HistoryIndex == -1 || g_HistoryIndex == 0)
    return;

  _LoadPage(*g_History.at(--g_HistoryIndex),g_WebHTML);
}

void Input_Callback(Fl_Widget * w, void * data)
{
  Fl_Input * i = (Fl_Input*)w;
  QString str  =  i->value();
  History hobj;

  // if old and new strings match, must have been a CR
  if(strcmp((char *)str,oldstr) == 0)
  {
    oldstr[0] = 0x00;
    hobj.AddToHistoryList(str);
    hindex = 1;
    LoadPage(str,g_WebHTML);
  }

  // if there's a space in str, set a history entry
  else if(strstr((char *)str," "))
  {
    i->value(hobj.GetHistoryEntry(hindex));
    i->set_changed();
    i->mark(256);
    strcpy(oldstr,hobj.GetHistoryEntry(hindex));
    if(hobj.GetHistoryEntry(++hindex) == NULL)
      hindex = 0;
  }

  // must be an updated string - save it
  else
  {
    strcpy(oldstr,(char *)str);
  }
}

VMLAppWindow * make_window() 
{
  char buf[256];

  g_XOffset = 0;
  g_YOffset = 0;
  g_ScreenX = 640;
  g_ScreenY = 480;


  VMLAppWindow * w;
  { 
    VMLAppWindow * o = new VMLAppWindow(g_AppX + g_XOffset,
					g_AppY + g_YOffset,
					APPWIDTH,
					APPHEIGHT);
    g_AppWindow = o;

    w = o;

    { 

      HTML_WIDGET * v;

      o->begin();
      o->show();

      // This is an ugly ugly hack .. because of broken parenting
      // in FLNX, we need to make sure the HTML Widget is added to
      // it's parent before it is mapped so the window manager
      // doesn't get ahold of it and do something evil. We do this
      // by passing in our top-level widget in the flags field
      // see htmlview.cpp for the other side of the disgusting piece
      // of code.

      v = new HTML_WIDGET(0,0,(int)o);
      v->setGeometry(SCROLLPAD,TITLEHEIGHT,WIDTH,HEIGHT);
      o->add(v);
      
      g_Conn->Connect(v);
      g_HTML = v;
    }
    g_AppWindow = o;
    o->end();
  }


  g_PinkWindow = new WMPinkWindow();
  g_PinkWindow->end();


  //////////////////////////////////////////////////////

  { 
    VMLAppWindow * o = new VMLAppWindow(g_AppX + g_XOffset,
					g_AppY + g_YOffset,
					APPWIDTH,
					APPHEIGHT);

    { 
      Fl_Button* o = Forward = new Fl_Button(30, 3, 25, 25);
      o->box(FL_FLAT_BOX);
#ifdef _NANOX
      o->down_box(FL_WHITE_BOX);
#endif
      o->callback(Forward_Callback,0);
      o->when(FL_WHEN_RELEASE);
      pixmap_forward.label(o);
    }
    { 
      Fl_Button* o = Backward = new Fl_Button(5, 3, 25, 25);
      o->box(FL_FLAT_BOX);
#ifdef _NANOX
      o->down_box(FL_WHITE_BOX);
#endif
      o->callback(Backward_Callback,0);
      o->when(FL_WHEN_RELEASE);
      pixmap_back.label(o);
    }

    { 
      Fl_Button* o = Reload = new Fl_Button(55, 3, 25, 25);
      o->box(FL_FLAT_BOX);
#ifdef _NANOX
      o->down_box(FL_WHITE_BOX);
#endif
      o->callback(Reload_Callback,0);
      o->when(FL_WHEN_RELEASE);
      pixmap_reload.label(o);
    }
    { 
      Fl_Input * o = g_URL = new Fl_Input(81, 5, APPWIDTH - 110 , 20, "");
      o->callback(Input_Callback,0);
      o->when(FL_WHEN_ENTER_KEY_CHANGED);
      g_Input = o;
    }
    
    {
      Fl_Button* o = g_Logo = new Fl_Button(APPWIDTH -24 ,5,20,20);
      o->box(FL_OFLAT_BOX);
      o->down_box(FL_OFLAT_BOX);
      o->callback(Return_Callback,0);
      o->when(FL_WHEN_RELEASE);
      pixmap_vml.label(o);
    }
    { 

      HTML_WIDGET * v;

      o->begin();
      o->show();

      // This is an ugly ugly hack .. because of broken parenting
      // in FLNX, we need to make sure the HTML Widget is added to
      // it's parent before it is mapped so the window manager
      // doesn't get ahold of it and do something evil. We do this
      // by passing in our top-level widget in the flags field
      // see htmlview.cpp for the other side of the disgusting piece
      // of code.

      v = new HTML_WIDGET(0,0,(int)o);
      v->setGeometry(SCROLLPAD,TITLEHEIGHT,WIDTH,HEIGHT);
      o->add(v);
      o->hide();
      g_Conn->Connect(v);
      g_WebHTML = v;
    }
    g_WebWindow = o;
    o->end();
  }

  // recover last entry from history and display the page
  History hobj;

  return w;
}

