/*******************************************************************************\
**
**	Module:	audio.cpp
**	Desc:	Handles all of the audio functionality for the desired channel (selection)
**	
**	Note:	Canniblized from Phat's vtest.cc, and stripped out of each of the following
**			modules into 1 to reduce the maintenance headache -- J Webb <jeffw@censoft.com>
**
\*******************************************************************************/

/* Feature test switches */


/* System header files */
#include <fcntl.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>
#include <unistd.h>

/* Local header files */
#include "audio.h"
#include "state.h"					// Probably a bad design to have this plugin module
#include "status.h"
									// tightly coupled with the main functionality

/* Typedef, macro, enum/struct/union definitions */


/* Global scope variables */


/* File scope variables */


/*******************************************************************************\
**
**	Function:	int SetAudio()
**	Desc:		Sets the audio to the desired level
**	Accepts:	int selection = The correct audio channel
**				int mute = Flag to determine if muting
**				int increment = increment value
**	Returns:	int; 0 on success -1 on error
**
\*******************************************************************************/
int
SetAudio(int selection, int mute, int increment) 
{
  static int mixdev[4] = { SOUND_MIXER_VIDEO,SOUND_MIXER_LINE,SOUND_MIXER_LINE,SOUND_MIXER_LINE1};
  
  char tmp[80];
  int mixer,rc,val; //audio,devmask;
  int volume;
  int muting;

   printf("Coming into SetAudio\n");

	  muting = mute;
  
  mixer = open( "/dev/mixer", O_RDONLY );


  if(mixer < 0)
    {
      VAPIStatusOutStr("open mixer failed\r\n"); //SOUND_MIXER_INFO
    }
  else
    {
      sprintf(tmp,"mixer = %X ", mixer);
      VAPIStatusOutStr(tmp);
      
      for(int i=0; i<4; i++) {
	val = 0x0000;
	rc = ioctl(mixer, _IOC(_IOC_WRITE|_IOC_READ,'M',mixdev[i],sizeof(int)), &val);
	if(rc < 0)
	  {
	    sprintf(tmp,"(%d)=_IOC_WRITE|_IOC_READ ", mixdev[i]);
	    VAPIStatusOutStr(tmp);	
	  }
	rc = ioctl(mixer, _IOC(_IOC_READ,'M',mixdev[i],sizeof(int)), &val);
	if(rc < 0)
	  {
	    sprintf(tmp,"(%d)=_IOC_READ ", mixdev[i]);
	    VAPIStatusOutStr(tmp);	
	    }
	sprintf(tmp,"(%d)=%X ", mixdev[i], val);
	VAPIStatusOutStr(tmp);
      }
      
	  /* Get the current volume */
	  volume = state_volumes[selection];
		  volume += increment;

	  /* Check for volume bounds (0-99) */
	  if (volume > 99)
		  volume = 99;
	  else if (volume < 1)
		  volume = 1;
	  
	  /* Alter the volume state value */
	  state_volumes[selection] = volume;

      val = (muting) ? 0 : ((volume << 8) + volume);

 rc = ioctl(mixer, _IOC(_IOC_WRITE|_IOC_READ,'M',mixdev[selection],sizeof(int)), &val);
      if(rc < 0)
	{
	  sprintf(tmp,"(%d)=_IOC_WRITE|_IOC_READ ", mixdev[selection]);
	  VAPIStatusOutStr(tmp);	
	}
      rc = ioctl(mixer, _IOC(_IOC_READ,'M',mixdev[selection],sizeof(int)), &val);
      if(rc < 0)
	{
	  sprintf(tmp,"(%d)=_IOC_READ ", mixdev[selection]);
	  VAPIStatusOutStr(tmp);	
	}
      sprintf(tmp,"(%d)=%X ", mixdev[selection], val);
      VAPIStatusOutStr(tmp);
      
      VAPIStatusOutStr("\r\n");	
      close(mixer);
    }
	printf("Getting out ot setaudio\n");
  return 0;
}
