/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: dvdplugin.cpp                                              *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "audio.h"
#include "cmdblock.h"
#include "cmdutil.h"
#include "dispatcher.h"
#include "errorbase.h"
#include "sharedclient.h"
#include "srcmpeg.h"
#include "state.h"



/* NSC middleware api */
#include <anlg_video.h>

/*
**
** Local Constant Definitions
**
*/
#define TIMEOUT 1
#define WRAPPER_LOCK_FILE	"/var/lock/dvdplayer.lock"



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/
typedef struct
{
  char *cmd;
  int (*func)(int argc, char *argv[]);
}CmdEntry;



/*
**
** Local Variable Declarations
**
*/
static char *plugin_name = "DVD Player and Drive Support";
static char *plugin_version = "1.0.0";
static char *plugin_keyword = "dvd";


/*
**
**
**
*/
static int loadFrame(void)
{
  int tick;

  // make sure cmd_block is idle
  if(cmd_block->status != WMSTATUS_IDLE)
    return(ERRORBASE_DVD + 10);

  // issue the stop command
  cmd_block->command = WMCMD_VIEWER_FRAME_LOAD;
  strcpy(cmd_block->parm1,"Dialog");
  strcpy(cmd_block->parm2,"dialogdvd.html");
  cmd_block->status = WMSTATUS_START;

  // wait for the command to finish
  tick = time(NULL);
  while(1)
  {
    if(cmd_block->status == WMSTATUS_DONE)
      break;
    usleep(1000);
    if((tick + TIMEOUT) <= time(NULL))
    {
      cmd_block->command = WMCMD_NONE;
      cmd_block->status = WMSTATUS_IDLE;
      //      return(ERRORBASE_DVD + 11);
      return(0);
    }
  }
  cmd_block->command = WMCMD_NONE;
  cmd_block->status = WMSTATUS_IDLE;
  
  return(0);
}



/*
**
** Command: source
**
*/
static int sourceFunc(int argc, char *argv[])
{
	int					rc = 0;											// Generic result code
	int					tmp;
  printf("DVD Source function stub\n");

  // Issue the call for setting up sound, etc
	// TODO:
	// 	The VAPISrcDVDSelect() call will soon be deprecated.  It is only in use to allow the sound
	// 	to correctly follow the video input device.  But will be replaced when NSC gets the audio
	// 	functionality into their middleware apis.

  if (VAPISrcMPEGSelect() != 0)
  	return (ERRORBASE_DVD + 0);

  	VMW_AVS_VideoInit();

	// Hack to get the dvd in the right decoder mode on the first pass through -- 
	// Don't know why this works, but it is in NSC 'win' stb integration test app
	
	VMW_OVL_EnableOverlay(0x00f000f0, 0x00f0f0f0, 0);

	rc = VIDEO_INPUT_MPEG_DECODER;
	VMW_AVS_SetCurrentVideoInput(&rc);

	VMW_AVS_VideoClose();
	SetAudio(AV_CHAN_MPEG,0,0);

		strcpy(state_source, plugin_keyword);
		loadFrame();
	
  return(0);
}

/*
**
** Command: eject
**
*/
static int ejectFunc(int argc, char *argv[])
{
  printf("DVD Eject function stub\n");
  return(0);
}



/*
**
** Command: stop
**
*/
static int stopFunc(int argc, char *argv[])
{
  	printf("DVD Stop function stub\n");
	return (WCMDU_ProcSimpleCmd(WMCMD_DVD_STOP, ERRORBASE_DVD + 2));
}



/*
**
** Command: play
**
*/
static int playFunc(int argc, char *argv[])
{
	printf("DVD Play function stub\n");
  
  // Ensure that the wrapper application is running
  if (!WCMDU_WrAppRunning(WRAPPER_LOCK_FILE))
  {
	  pid_t				pid;										// Pid of the process

	  fprintf(stdout, "dvdplugin:DEBUG exec'ing /bin/dvdplayer\n");
	  if ( (pid = fork()) == 0)
	  {
		  // Child process
		  execl("/bin/dvdplayer", "dvdplayer", "", NULL);
		  return (ERRORBASE_DVD + 3);
	  } // end of if 

	  WCMDU_UpdAppLckFile(pid, WRAPPER_LOCK_FILE);
	  return (0);
  } // end of if 
  else
  {
	return (WCMDU_ProcSimpleCmd(WMCMD_DVD_PLAY, ERRORBASE_DVD + 4));
  } // end of else
} // end of playFunc()


/*
**
** Command: seek
**
*/
static int seekFunc(int argc, char *argv[])
{
	// Not implemented yet
  printf("DVD Seek function stub\n");
  return(0);
}



/*
**
** Command: forward scan
**
*/
static int ffwdFunc(int argc, char *argv[])
{
  printf("DVD Forward Scan function stub\n");
  return (WCMDU_ProcSimpleCmd(WMCMD_DVD_FWD, ERRORBASE_DVD + 6));
}



/*
**
** Command: reverse scan
**
*/
static int rewFunc(int argc, char *argv[])
{
  printf("DVD Reverse Scan function stub\n");
  return(WCMDU_ProcSimpleCmd(WMCMD_DVD_REV, ERRORBASE_DVD + 8));
}



/*
**
** Command: pause
**
*/
static int pauseFunc(int argc, char *argv[])
{
  printf("DVD Pause function stub\n");
  return(WCMDU_ProcSimpleCmd(WMCMD_DVD_PAUSE, ERRORBASE_DVD + 10));
}

/*
**
** Command List
**
*/
static CmdEntry cmdlist[] =
{
  "source",sourceFunc,
  "eject",ejectFunc,
  "stop",stopFunc,
  "play",playFunc,
  "seek",seekFunc,
  "ffwd",ffwdFunc,
  "rew",rewFunc,
  "pause",pauseFunc,
  NULL,NULL
};



/*
**
** This function is called once, just after the plugin has been loaded
** and all plugin entrypoints have been registered. This function will
** be called before any other plugin entrypoint functions are called.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginStartup(void)
{
  return(0);
}



/*
**
** This function is called once, just before the plugin is unloaded.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginShutdown(void)
{
  return(0);
}



/*
**
** This function is called to recover the plugin's name/description.
**
** The "result" parameter is a pointer to the buffer where the plugin's
** name/description will be stored.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginName(char *result)
{
  strcpy(result,plugin_name);
  return(0);
}



/*
**
** This function is called to recover the plugin's internal version number.
**
** The "result" parameter is a pointer to the buffer where the plugin's
** internal version number will be stored.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginVersion(char *result)
{
  strcpy(result,plugin_version);
  return(0);
}



/*
**
** This function is called to recover the plugin's command keyword.
**
** The "result" parameter is a pointer to the buffer where the plugin's
** command keyword will be stored.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginKeyword(char *result)
{
  strcpy(result,plugin_keyword);
  return(0);
}



/*
**
** This function is called to display this plugin's command set.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginHelp(void)
{
  printf("   %s source\n",plugin_keyword);
  printf("   %s eject\n",plugin_keyword);
  printf("   %s stop\n",plugin_keyword);
  printf("   %s play\n",plugin_keyword);
  printf("   %s seek (-)n\n",plugin_keyword);
  printf("   %s ffwd\n",plugin_keyword);
  printf("   %s rew\n",plugin_keyword);
  printf("   %s pause\n", plugin_keyword);
  return(0);
}



/*
**
** This function is called to execute one of the plugin's commands.
**
** The "argc" parameter specifies the number of string parameters
** found in the argv array. The "argv" parameter is an array of
** pointers to the command and it's parameters.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginCommand(int argc, char *argv[])
{
  int count,result;

  
  // scan the list of commands for a match
  count = 0;
  while(1)
  {
    // check for end of list (indicates unknown command)
    if(cmdlist[count].cmd == NULL)
    {
      result = ERRORBASE_DVD + 0;
      break;
    }

    // check for command match
    if(strcmp(cmdlist[count].cmd,argv[0]) == 0)
    {
      result = (cmdlist[count].func)(argc - 1,&argv[1]);
      break;
    }

    // bump to next command
    ++count;
  }

  // return result and exit
  return(result);
}















