/*
**
** Imported "Include" Files
**
*/
//#include <direct.h>
//#include <io.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
//#include <sys/types.h>
//#include <sys/stat.h>



/*
**
** Local Constant Definitions
**
*/
#define COPY_BUFFER_SIZE 32768



/*
**
** This function will copy and rename the specified file to the 
** specified directory.
**
** The "dstfile" parameter is a pointer to a buffer that contains the
** destination filename and optional path. The "srcfile" parameter is
** a pointer to a buffer that contains the source filename and optional
** path. The "progress" parameter is a pointer to a function called
** after each block of data is copied. The "progress_type" parameter
** specifies the contents of the string returned by the progress()
** function.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** The "progress" function is called after each block of data is copied.
** Normally, this function should return '0'. To cancel the session,
** return a non-zero value. To disable the progress function, set this
** parameter to NULL.
**
*/

int copyFileToFile(char *dstfile,char *srcfile)
{
  FILE *sfp,*dfp;
  int result,total;
  unsigned char *data;

  /* open the source file */
  if((sfp = fopen(srcfile,"rb")) == NULL)
    return(-1);

  /* open the destination file */
  if((dfp = fopen(dstfile,"wb")) == NULL)
  {
    fclose(sfp);
    return(-2);
  }

  /* allocate memory for the copy buffer */
  data = (unsigned char *)malloc(COPY_BUFFER_SIZE);
  if(data == NULL)
  {
    fclose(dfp);
    fclose(sfp);
    return(-3);
  }

  /* copy the file */
  total = 0;
  while(1)
  {
    result = fread(data,1,COPY_BUFFER_SIZE,sfp);
    if(result == 0)
      break;
    fwrite(data,1,result,dfp);
  }

  /* do housekeeping */
  free(data);
  fclose(dfp);
  fclose(sfp);

  /* exit with no errors */
  return(0);
}
