/*******************************************************************************\
**
**	Program:	idbugvwr.cpp
**	Desc:		A cannabolized version of the microwindows/demos/mwin/demo3.c
**				That is specialized for the distinct purpose of displaying an
**				image at a given x,y coordinate.
**	History:	05/15/01		J Webb <jeffw@censoft.com>
**								Initial Version
**
\*******************************************************************************/


// Feature test switches


// System Header files
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Local header files (including NSC Middleware api headers)
#include <anlg_video.h>
#include <vmwdvdapi.h>
#define MWINCLUDECOLORS
#include "nano-X.h"

// Typedef, macro, class/enum/struct/union definition
typedef struct
{
	int					achan,										// alpha channel
						alvl,										// alpha level
						xPos,										// UL X Value of AB Window
						yPos;										// UL Y Value of AB Window
	char				*abImage;									// Pointer to the abImage value
} abInfo;

#define		SAFETITLE_LEFT		-1									// Align with Safe title left x boundary
#define		SAFETITLE_RIGHT		-2									// Align with Safe title right x boundary
#define		SAFETITLE_TOP		-3									// Align with safe title top boundary
#define		SAFETITLE_BOTTOM	-4									// Align with safe title bottom boundary


// Global scope variable


// Local scope variables
// NOTE:	These are only "global" to allow for the atexit handler to
// 			properly perform the necessary clean-up on termination
GR_WINDOW_ID			imgwin = 0;								// Image window
GR_GC_ID				imgGC = 0;									// Image's GC
GR_IMAGE_ID				imgID = 0;									// Image id
static sig_atomic_t		exitflag = 0;								// Global exit flag
static abInfo			abinfo;										// Needed to be global for exitHandler

// ANSI Function prototypes
static void CalcSafeTitle(VIDEO_FORMAT *pVFmt, int *xadj, int *yadj);
void exitHandler(void);
static void SetFromSafeTitle(GR_IMAGE_INFO *pImgInfo, int *xpos, int *ypos);
void sigHusr1(int signo);
static void vwrUsage(char *prcName);


int
main(int argc, char **argv)
{
	int					rc;											// Result code
	GR_EVENT			event;										// Event mask
	GR_IMAGE_INFO		info;										// Info

	// Since this process is designed to be exec'd by another process, the
	// order of the parameters will be set, which avoids the overhead associated
	// with the getopt() type parameter parsing stuff.  Arguments are
	// arg1 = xposition
	// arg2 = yposition
	// arg3 = alpha channel
	// arg4 = alpha level
	// arg5 = filename
	if (argc != 6)
	{
		vwrUsage(argv[0]);
		exit(EXIT_FAILURE);
	} // end of if 

	// Parse & Validate the arguments -- where applicable
	abinfo.xPos = atoi(argv[1]);
	abinfo.yPos = atoi(argv[2]);
	abinfo.achan = atoi(argv[3]);
	abinfo.alvl = atoi(argv[4]);

	if ( (abinfo.abImage = argv[5]) == NULL)
	{
		fprintf(stderr, "Image name %s invalid.\n", argv[5]);
		exit (EXIT_FAILURE);
	} // end of if

	// Set up the SIGUSR1 signal handler
	{
		struct sigaction	susr1;									// Signal handler for sigusr1 signal

		memset(&susr1, 0, sizeof(susr1));
		susr1.sa_handler = sigHusr1;
		sigaction(SIGUSR1, &susr1, (struct sigaction *)0);
	} // end of memory
	
	// Open the graphics
	if (GrOpen() < 0)
	{
		fprintf(stderr, "Unable to open graphics.\n");
		exit (EXIT_FAILURE);
	} // end of if 
	atexit(exitHandler);

	// Load the image from file
	if ( (imgID = GrLoadImageFromFile(abinfo.abImage, 0)) <= 0)
	{
		fprintf(stderr, "GrLoadImageFromFile(%s) failed.\n", abinfo.abImage);
		exit(EXIT_FAILURE);
	} // end of if 

	// Determine the size of the image and the enclosing window
	GrGetImageInfo(imgID, &info);

	// Set up the alpha channel
	if ( (rc = VMW_AVS_VideoInit()) != VMW_OK)
	{
		fprintf(stdout, "VMW_AVS_VideoInit() failed\n");
		exit (EXIT_FAILURE);
	} // end of if 
	
	// Verify the positions
	if (abinfo.xPos < 0 || abinfo.yPos < 0)
	{
		// A safe title boundary is desired on at least one axis.
		SetFromSafeTitle(&info, &abinfo.xPos, &abinfo.yPos);
	} // end of if 
	
	if ( (rc = VMW_OVL_EnableOverlay(0x00f000f0, 0x00f0f0f0, 1)) != VMW_OK)
	{
		fprintf(stdout, "VMW_OVL_EnableOverlay() failed\n");
		exit (EXIT_FAILURE);
	} // end of if
		
	if ( (rc = VMW_OVL_EnableAlphaBlendWindow(abinfo.xPos, abinfo.yPos, info.width, info.height,
									abinfo.alvl, 0, abinfo.achan)) != VMW_OK)
	{
		fprintf(stdout, "VMW_OVL_EnableAlphaBlendWindow() returned=%d.\n", rc);
		exit (EXIT_FAILURE);
	} // end of if
	VMW_AVS_VideoClose();

	// Create the window and set the events
	imgwin = GrNewWindow(GR_ROOT_WINDOW_ID, abinfo.xPos, abinfo.yPos, info.width, info.height,
										  0, BLACK, BLACK);

	GrSelectEvents(imgwin, GR_EVENT_MASK_CLOSE_REQ | GR_EVENT_MASK_EXPOSURE);
	GrMapWindow(imgwin);
	imgGC = GrNewGC();
	GrSetGCForeground(imgGC, WHITE);

	while (!exitflag)
	{
		GrGetNextEvent(&event);
		switch (event.type)
		{
			case GR_EVENT_TYPE_CLOSE_REQ:
				exitflag = 1;
				break;
			case GR_EVENT_TYPE_EXPOSURE:
				GrDrawImageToFit(imgwin, imgGC, 0, 0, info.width, info.height, imgID);
				break;
		} // end of switch
	}

	exit (EXIT_SUCCESS);
} // end of main

/*******************************************************************************\
**
**	Function:	static void CalcSafeTitle()
**	Desc:		Determines the safe title area for a given mode
**	Accepts:	VIDEO_FORMAT *pVFmt = Ptr to the current video format
**				int *xadj = Ptr to store amount to adjust x
**				int *yadj = Ptr to store amount to adjust y
**	Returns:	Nothing (void)
**
\*******************************************************************************/
static void
CalcSafeTitle(VIDEO_FORMAT *pVFmt, int *xadj, int *yadj)
{
	// TODO: determine the viewable viewing area for H/V
	
	*xadj = (int)(pVFmt->fmt.pix.width * 0.1);
	*yadj = (int)(pVFmt->fmt.pix.height * 0.1);

	return;
} // end of CalcSafeTitle()

/*******************************************************************************\
**
**	Function:	void exitHandler()
**	Desc:		Exit handler for this routine.  Responsible for clean-up prior
**				to termination
**	Accepts:	Nothing (void)
**	Returns:	Nothing (void)
**
\*******************************************************************************/
void
exitHandler(void)
{
	// Properly destroy contents of this processes windows
	GrDestroyGC(imgGC);
	GrFreeImage(imgID);
	GrCloseWindow(imgwin);
	
	// Shutdown microwindows
	GrClose();

	// Disable the alpha blend window
	VMW_AVS_VideoInit();
	VMW_OVL_DisableAlphaWindow(abinfo.achan);
	VMW_AVS_VideoClose();

	return;
} // end of exitHandler()

/*******************************************************************************\
**
**	Function:	static void SetFromSafeTitle()
**	Desc:		Calculates the safetitle area for the current resoultion and sets
**				either the x, y or both Upper Left coordinates accordingly.
**	Accepts:	GR_IMAGE_INFO *pImgInfo = Ptr to the image info
**				int *xpos = Ptr to the xpos variable
**				int *ypos = Ptr to the ypos variable
**	Returns:	Nothing (void)
**
\*******************************************************************************/
static void
SetFromSafeTitle(GR_IMAGE_INFO *pImgInfo, int *xpos, int *ypos)
{
	int					xdelta = 0,									// Delta in x
						ydelta = 0;									// Delta in y
	VIDEO_FORMAT		sVidFmt;									// Video format
	
	// Get the video resolution
	memset(&sVidFmt, 0, sizeof(VIDEO_FORMAT));
	VMW_AVC_GetCaptureFormat(&sVidFmt);
	
	// Calculate the safe title area of from this resoltion
	CalcSafeTitle(&sVidFmt, &xdelta, &ydelta);

	// Determine the x values
	switch(*xpos)
	{
		case SAFETITLE_LEFT: *xpos = xdelta; break;
		case SAFETITLE_RIGHT: *xpos = sVidFmt.fmt.pix.width - (xdelta + pImgInfo->width); break;
	} // end of switch

	// Determine the y values
	switch(*ypos)
	{
		case SAFETITLE_TOP: *ypos = ydelta; break;
		case SAFETITLE_BOTTOM: *ypos = sVidFmt.fmt.pix.height - (ydelta + pImgInfo->height); break;
	} // end of switch

	return;
} // end of SetFromSafeTitle()

/*******************************************************************************\
**
**	Function:	void sigHusr1()
**	Desc:		Signal handling function for receipt of the sigusr1 signal --- which
**				effectively performs a shutdown.
**	Accepts:	int signo = Signal number that generated the branch into here
**	Returns:	Nothing (void)
**
\*******************************************************************************/
void
sigHusr1(int signo)
{
	if (signo == SIGUSR1)
		exitflag = 1;
} // end of sigHusr1 

/*******************************************************************************\
**
**	Function:	static void vwrUsage()
**	Desc:		Displays the expected usage message for this process
**	Accepts:	char *prcName = Name this process was invoked as
**	Returns:	Nothing (void)
**
\*******************************************************************************/
static void
vwrUsage(char *prcName)
{
	char				*p;											// Base pointer

	if ( (p = strrchr(prcName, '/')) == NULL)
		p = prcName;
	else
		p++;

	fprintf(stdout, "\n%s:  Basic microwindows image viewer.\n", p);
	fprintf(stdout, "\tSyntax: %s x-val y-val image_path.\n\n", p);
	
	return;
} // end of vwrUsage()
