/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: plugin.cpp                                                 *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <dirent.h>
#include <dlfcn.h>
#include <stdio.h>
#include <string.h>
#include "errorbase.h"
#include "plugin.h"



/*
**
** Global Variable Definitions
**
*/
PluginEntry plugin_list[MAX_PLUGINS];
int plugin_list_len;



/*
**
** Local Constant Definitions
**
*/



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/



/*
**
** Local Variable Declarations
**
*/



/*
**
** This function will load and register all plugins found in the
** specified directory.
**
** The "dir" parameter is a pointer to the buffer that contains the
** directory to be searched for plugins.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int loadPlugins(char *dir)
{
  DIR *dbase;
  PluginEntry pentry;
  char buf[256];
  struct dirent *dentry;

  // initialize the global variables
  plugin_list_len = 0;

  // open the specified keymap directory
  dbase = opendir(dir);
  if(dbase == NULL)
    return(ERRORBASE_PLUGIN + 0);

  // load and register all plugins
  while(1)
  {
    // recover the next directory entry
    dentry = readdir(dbase);
    if(dentry == NULL)
      break;

    // skip current, parent, and sub directories
    if(strcmp(dentry->d_name,".") == 0)
      continue;
    if(strcmp(dentry->d_name,"..") == 0)
      continue;

    // see if this entry is a potential match
    if(strstr(dentry->d_name,".wmp") == NULL)
      continue;

    // potential match found - attempt to load
    sprintf(buf,"%s/%s",dir,dentry->d_name);
    pentry.handle = dlopen(buf,RTLD_NOW);
    if(pentry.handle == NULL)
    {
      printf("%s\n",dlerror());
      continue;
    }

    // potential match found - attemp to register entrypoints
    dlerror();
    pentry.startupFunc =
      (int (*)(void))dlsym(pentry.handle,"pluginStartup");
    pentry.shutdownFunc =
      (int (*)(void))dlsym(pentry.handle,"pluginShutdown");
    pentry.nameFunc = 
      (int (*)(char *))dlsym(pentry.handle,"pluginName");
    pentry.versionFunc = 
      (int (*)(char *))dlsym(pentry.handle,"pluginVersion");
    pentry.keywordFunc = 
      (int (*)(char *))dlsym(pentry.handle,"pluginKeyword");
    pentry.helpFunc = 
      (int (*)(void))dlsym(pentry.handle,"pluginHelp");
    pentry.commandFunc = 
      (int (*)(int, char *[]))dlsym(pentry.handle,"pluginCommand");
    if((pentry.startupFunc == NULL) ||
       (pentry.shutdownFunc == NULL) ||
       (pentry.nameFunc == NULL) ||
       (pentry.versionFunc == NULL) ||
       (pentry.keywordFunc == NULL) ||
       (pentry.helpFunc == NULL) ||
       (pentry.commandFunc == NULL))
    {
      dlclose(pentry.handle);
      continue;
    }

    // valid plugin - add it to the list
    strcpy(pentry.filename,dentry->d_name);
    plugin_list[plugin_list_len++] = pentry;

    // call this plugin's startup function
    (pentry.startupFunc)();

    // display plugin name
    //(pentry.nameFunc)(buf);
    //printf("Plugin: \"%s\"\n",buf);
  }
  closedir(dbase);

  // exit with no errors
  return(0);
}



/*
**
** This function will unload all plugins previously loaded by the
** "pluginLoad()" function and free all associated memory.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int unloadPlugins(void)
{
  int count;

  // unload (close) all plugins
  count = 0;
  do
  {
    (plugin_list[count].shutdownFunc)();
    dlclose(plugin_list[count].handle);
  }while(++count < plugin_list_len);

  // exit with no errors
  return(0);
}
