/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: sharedmem.cpp                                              *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "cmdblock.h"



/*
**
** Global Variable Declarations
**
*/
WMCmdBlock *cmd_block;



/*
**
** Local Constant Definitions
**
*/



/*
**
** Local Variable Declarations
**
*/




/*
**
** This function will create a new block of shared memory. The newly
** created block will be cleared, the command will be set to "none",
** and the status set to "idle".
**
** If successful, '0' is returned. If an error occurs during function
** exeuction, a non-zero value is returned that describes the error.
**
** The shared memory handle is written to a file which is used by
** other clients to attach to the shared memory. The actual location
** of the file is specified in the "cmdblock.h" file.
**
*/
int clientMemoryCreate(void)
{
  FILE *fp;
  char buf[256];
  int smid;
  struct shmid_ds sminfo;

  // attempt to recover the shared memory ID
  if((fp = fopen(SM_FILENAME,"r")) != NULL)
  {
    if(fgets(buf,256,fp) != NULL)
    {
      smid = atoi(buf);
      if(shmctl(smid,IPC_STAT,&sminfo) == 0)
      {
#ifdef NEVER
	printf("Shared memory already mapped: id:%d, size:%d, attached:%d\n",
	       smid,
	       sminfo.shm_segsz,
	       sminfo.shm_nattch);
#endif
	fclose(fp);
	return(0);
      }
    }
    fclose(fp);
  }

  // block does not exist - create one
  smid = shmget(IPC_PRIVATE,sizeof(WMCmdBlock),0777);
  if(smid == -1)
    return(-1);

  // block created - now attach to it
  cmd_block = (WMCmdBlock *)shmat(smid,0,0);
  if(cmd_block == ((WMCmdBlock *)-1))
    return(-2);

  // setup initial block values
  cmd_block->status = WMSTATUS_IDLE;
  cmd_block->command = WMCMD_NONE;
  cmd_block->parm1[0] = 0x00;
  cmd_block->parm2[0] = 0x00;
  cmd_block->parm3 = 0;
  cmd_block->parm4 = 0;

  // detach from block
  shmdt(cmd_block);

  // write out shared memory ID
  if((fp = fopen(SM_FILENAME,"w")) == NULL)
    return(-3);
  fprintf(fp,"%d\n",smid);
  if(fclose(fp) != 0)
    return(-4);

  // exit with no errors
  return(0);
}



/*
**
** This function will attach to the WebMedia shared memory command
** block and initialize the "cmd_block" pointer.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
*/
int clientMemoryAttach(void)
{
  FILE *fp;
  char buf[256];
  int id;

  // open the state file
  if((fp = fopen(SM_FILENAME,"r")) == NULL)
  {
    printf("Unable to open \"%s\"...\n",SM_FILENAME);
    return(-1);
  }

  // recover the shared memory ID
  if(fgets(buf,256,fp) == NULL)
  {
    printf("Unable to read \"%s\"...\n",SM_FILENAME);
    return(-2);
  }
  id = atoi(buf);

  // attach to the shared memory
  cmd_block = (WMCmdBlock *)shmat(id,0,0);
  if(cmd_block == ((WMCmdBlock *)-1))
  {
    printf("Unable to attach to shared memory...\n");
    return(-3);
  }

  // exit with no errors
  return(0);
}



/*
**
** This function will detach from the WebMedia shared memory command
** block and free all allocated memory.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int clientMemoryDetach(void)
{
  shmdt(cmd_block);
  return(0);
}
