/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: statemodule.cpp                                           *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <stdio.h>
#include <stdlib.h>
#include "dispatcher.h"
#include "errorbase.h"
#include "state.h"



/*
**
** Local Constant Definitions
**
*/



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/
typedef struct
{
  char *cmd;
  int (*func)(int argc, char *argv[]);
}CmdEntry;



/*
**
** Local Variable Declarations
**
*/
static char *plugin_name = "System State Management";
static char *plugin_version = "1.0.0";
static char *plugin_keyword = "state";
static char cmdbuf[256];



/*
**
** Command: restore
**
*/
static int restoreFunc(int argc, char *argv[])
{
  int result;

  // set the video source
  sprintf(cmdbuf,"%s source",state_source);
  result = dispatcher(cmdbuf);
  if(result)
    return(result);

  // set the tuner channel
  sprintf(cmdbuf,"tuner channel %d",state_channel);
  result = dispatcher(cmdbuf);
  if(result)
    return(result);

  // set the vidwin position and size
  sprintf(cmdbuf,"vidwin position %d %d",state_vidwin_xpos,state_vidwin_ypos);
  result = dispatcher(cmdbuf);
  if(result)
    return(result);
  sprintf(cmdbuf,"vidwin size %d %d",state_vidwin_width,state_vidwin_height);
  result = dispatcher(cmdbuf);
  if(result)
    return(result);

  // exit with no errors
  return(0);
}



/*
**
** Command: write
**
*/
static int writeFunc(int argc, char *argv[])
{
  // check for proper number of parameters
  if(argc != 9)
    return(ERRORBASE_STATEMOD + 0);

  // extract the parameters
  strcpy(state_source,argv[0]);
  state_channel = atoi(argv[1]);
  state_channel_max = atoi(argv[2]);
  state_channel_min = atoi(argv[3]);
  state_vidwin_xpos = atoi(argv[4]);
  state_vidwin_ypos = atoi(argv[5]);
  state_vidwin_width = atoi(argv[6]);
  state_vidwin_height = atoi(argv[7]);
  strcpy(state_html_base,argv[8]);

  // exit with no errors
  return(0);
}



/*
**
** Command List
**
*/
static CmdEntry cmdlist[] =
{
  "restore",restoreFunc,
  "write",writeFunc,
  NULL,NULL
};



/*
**
** This function will initialize the state module and prepare it
** for subsequent operations.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginStartup(void)
{
  return(0);
}



/*
**
** This function will closeout the state module and shut it down.
**
** If successful, '0' is returned. If an error occurs during function
** exeuction, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginShutdown(void)
{
  return(0);
}



/*
**
** This function will return the subsystem identification string
** for the state module ("state").
**
** The "result" parameter is a pointer to the buufer where the
** subsystem identification string will be stored.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginKeyword(char *result)
{
  strcpy(result,plugin_keyword);
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginName(char *result)
{
  strcpy(result,plugin_name);
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginVersion(char *result)
{
  strcpy(result,plugin_version);
  return(0);
}



/*
**
** This function will display the commands available for the state
** module.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginHelp(void)
{
  printf("   %s restore\n",plugin_keyword);
  printf("   %s write\n",plugin_keyword);
  return(0);
}



/*
**
** This function will execute the specified state module command.
**
** The "argc" parameter specifies the number of string parameters
** found in the argv array. The "argv" parameter is an array of
** pointers to the command and it's parameters.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginCommand(int argc, char *argv[])
{
  int count,result;

  // scan the list of commands for a match
  count = 0;
  while(1)
  {
    // check for end of list (indicates unknown command)
    if(cmdlist[count].cmd == NULL)
    {
      result = ERRORBASE_STATEMOD + 1;
      break;
    }

    // check for command match
    if(strcmp(cmdlist[count].cmd,argv[0]) == 0)
    {
      result = (cmdlist[count].func)(argc - 1,&argv[1]);
      break;
    }

    // bump to next command
    ++count;
  }

  // return result and exit
  return(result);
}
