/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: svideoplugin.cpp                                           *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "audio.h"
#include "errorbase.h"
#include "sharedclient.h"
#include "srcsvideo.h"
#include "state.h"

#include <anlg_video.h>


/*
**
** Local Constant Definitions
**
*/
#define TIMEOUT 1



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/
typedef struct
{
  char *cmd;
  int (*func)(int argc, char *argv[]);
}CmdEntry;



/*
**
** Local Variable Declarations
**
*/
static char *plugin_name = "S-Video (Y/C) Audio/Video";
static char *plugin_version = "1.0.0";
static char *plugin_keyword = "svideo";



/*
**
**
**
*/
static int loadFrame(void)
{
  int tick;

  // make sure cmd_block is idle
  if(cmd_block->status != WMSTATUS_IDLE)
    return(ERRORBASE_SVIDEO + 10);

  // issue the stop command
  cmd_block->command = WMCMD_VIEWER_FRAME_LOAD;
  strcpy(cmd_block->parm1,"Dialog");
  strcpy(cmd_block->parm2,"dialogsvideo.html");
  cmd_block->status = WMSTATUS_START;

  // wait for the command to finish
  tick = time(NULL);
  while(1)
  {
    if(cmd_block->status == WMSTATUS_DONE)
      break;
    usleep(1000);
    if((tick + TIMEOUT) <= time(NULL))
    {
      cmd_block->command = WMCMD_NONE;
      cmd_block->status = WMSTATUS_IDLE;
      //      return(ERRORBASE_SVIDEO + 11);
      return(0);
    }
  }
  cmd_block->command = WMCMD_NONE;
  cmd_block->status = WMSTATUS_IDLE;
  
  return(0);
}



/*
**
** Command: source
**
*/
static int sourceFunc(int argc, char *argv[])
{
	int					svidrc = 0;									// Assume success

	// TODO:
	// 	This function will soon be deprecated and replaced by NSC middleware api calls.
  if(VAPISrcSvideoSelect() != 0)
    return(ERRORBASE_SVIDEO + 0);

	VMW_AVS_VideoInit();
	svidrc = VIDEO_INPUT_SVIDEO;
	VMW_AVS_SetCurrentVideoInput(&svidrc);
	VMW_OVL_EnableOverlay(0x00f000f0, 0x00f0f0f0, 0);
	VMW_AVS_VideoClose();
		strcpy(state_source,plugin_keyword);
		loadFrame();

  return(0);
}

/*******************************************************************************\
**
**	Function:	int volumeupFunc()
**	Desc:		Increments the volume on this channel by 1
**	Accepts:	int argc = Number of args
**				char *argv[] = Argument vector
**	Returns:	int; 0 on success, ERRORBASE value on error
**
\*******************************************************************************/
static int
volumeupFunc(int argc, char *argv[])
{
	printf("Lakshmi -- Getting into volumeupFunc\n");
	SetAudio(AV_CHAN_SVIDEO, 0, 1);
	printf("Lakshmi -- Getting out of volumeupFunc\n");
	return (0);
} /* end of volumeupFunc() */

/*******************************************************************************\
**
**	Function:	int volumedownFunc()
**	Desc:		Decrements the volume on this channel by 1
**	Accepts:	int argc = Number of args
**				char *argv[] = Argument vector
**	Returns:	int; 0 on success, ERRORBASE value on error
**
\*******************************************************************************/
static int
volumedownFunc(int argc, char *argv[])
{
	printf("Lakshmi -- Getting into volumedownFunc\n");
	SetAudio(AV_CHAN_SVIDEO, 0, -1);
	printf("Lakshmi -- Getting out volumedownFunc\n");

	return (0);
} /* end of volumedownFunc() */

/*******************************************************************************\
**
**	Function:	int volumemuteFunc()
**	Desc:		Mutes the volume on this audio channel
**	Accepts:	int argc = Number of args
**				char *argv[] =  Argument vector
**	Returns:	int; 0 on success, ERRORBASE value on error
**
\*******************************************************************************/
static int
volumemuteFunc(int argc, char *argv[])
{
	fprintf(stderr, "Muting svideo\n");
	SetAudio(AV_CHAN_SVIDEO, 1, 0);
	return (0);
} /* end of volumemuteFunc() */

/*******************************************************************************\
**
**	Function:	int volumeunmuteFunc()
**	Desc:		Un-mutes the volume on this audio channel
**	Acecpts:	int argc = Number of args
**				char *argv[] = Argument vector
**	Returns:	int; 0 on success, ERRORBASE value on error
**
\*******************************************************************************/
static int
volumeunmuteFunc(int argc, char *argv[])
{
	fprintf(stderr, "Unmuting svideo\n");
	SetAudio(AV_CHAN_SVIDEO, 0, 0);
	return (0);
} /* end of volumeunmuteFunc() */


/*
**
** Command List
**
*/
static CmdEntry cmdlist[] =
{
  "source",sourceFunc,
  "volup", volumeupFunc,
  "voldn", volumedownFunc,
  "mute", volumemuteFunc,
  "unmute", volumeunmuteFunc,
  NULL,NULL
};



/*
**
** This function will initialize the svideo module and prepare it
** for subsequent operations.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginStartup(void)
{
  return(0);
}



/*
**
** This function will closeout the svideo module and shut it down.
**
** If successful, '0' is returned. If an error occurs during function
** exeuction, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginShutdown(void)
{
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginName(char *result)
{
  strcpy(result,plugin_name);
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginVersion(char *result)
{
  strcpy(result,plugin_version);
  return(0);
}



/*
**
** This function will return the subsystem identification string
** for the svideo module ("svideo").
**
** The "result" parameter is a pointer to the buffer where the
** subsystem identification string will be stored.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginKeyword(char *result)
{
  strcpy(result,plugin_keyword);
  return(0);
}



/*
**
** This function will display the commands available for the 
** svideo module.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginHelp(void)
{
  printf("   %s source\n",plugin_keyword);
  printf("   %s mute\n",plugin_keyword);
  printf("   %s unmute\n",plugin_keyword);
  printf("   %s volup\n",plugin_keyword);
  printf("   %s voldn\n",plugin_keyword);
  return(0);
}



/*
**
** This function will execute the specified svideo module command.
**
** The "argc" parameter specifies the number of string parameters
** found in the argv array. The "argv" parameter is an array of
** pointers to the command and it's parameters.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginCommand(int argc, char *argv[])
{
  int count,result;

  // scan the list of commands for a match
  count = 0;
  while(1)
  {
    // check for end of list (indicates unknown command)
    if(cmdlist[count].cmd == NULL)
    {
      result = ERRORBASE_SVIDEO + 1;
      break;
    }

    // check for command match
    if(strcmp(cmdlist[count].cmd,argv[0]) == 0)
    {
      result = (cmdlist[count].func)(argc - 1,&argv[1]);
      break;
    }

    // bump to next command
    ++count;
  }

  // return result and exit
  return(result);
}
