/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: vidwinmodule.cpp                                            *
*                                                                      *
***********************************************************************/

/*******************************************************************************\
**
**	Note:	The functionality of this has been modified from the original intent,
**			for the purpose of NSC Middleware integration.
**			Assuming that their middleware cleans up after itself on application 
**			close, (thus closing any file descriptors and general clean up...),
**			there is now a vidwinctrl wrapper routine which sits and waits for 
**			certain commands on the shared memory segment, processing them from
**			this routine (and others).  This routine will ensure that the wrapper
**			application is always executing by starting it up, if need be
**
\*******************************************************************************/


/*
**
** Imported "Include" Files
**
*/
#include <ctype.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include "cmdblock.h"
#include "cmdutil.h"
#include "errorbase.h"
#include "overlay.h"
#include "srctuner.h"
#include "sharedclient.h"
#include "state.h"

#include <anlg_video.h>


/*
**
** Local Constant Definitions
**
*/
#define		TIMEOUT					5								// Time out value


/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/
typedef struct
{
  char *cmd;
  int (*func)(int argc, char *argv[]);
}CmdEntry;



/*
**
** Local Variable Declarations
**
*/
static char *plugin_name = "Scaleable, Positional Video Window";
static char *plugin_version = "1.0.0";
static char *plugin_keyword = "vidwin";

/*
**
**	ANSI C Function prototypes
**
**
*/
static int PostResize(int error);


/*******************************************************************************\
**
**	Function:	static int PostResize()
**	Desc:		Posts the resize command to the current enabled output source
**	Acecpts:	int error = The error to return;
**	Returns:	int; 0 on success, or the appropriate ERORRBASE_VIDWIN offset
**
**	Note:		This is hard-coded to return 0 (success) because it will fail
**				only if there is no process, and if that is the case, it just
**				needs to set the state correctly...
**
\*******************************************************************************/
static int
PostResize(int error)
{
	// This function posts a WMCMD_RESIZE command onto the shared memory command block
	// for processing by the appropriate client
	
	// Check for idle status
//	if (cmd_block->status != WMSTATUS_IDLE)
//		return (error -1);
//	
//	fprintf(stdout, "vidwinplugin:DEBUG Posting new window sizes %d %d %d %d.\n",
//				state_vidwin_xpos, state_vidwin_ypos, state_vidwin_width, state_vidwin_height);
//	cmd_block->command = WMCMD_VIDWIN_RESIZE;
//	sprintf(cmd_block->parm1, "%d,%d,%d,%d", state_vidwin_xpos, state_vidwin_ypos,
//				state_vidwin_width, state_vidwin_height);
//	cmd_block->status = WMSTATUS_START;
//	
//	return (WCMDU_DidCmdFinish(error));
	return (VMW_AVS_SetVideoWindow(state_vidwin_xpos, state_vidwin_ypos,
									state_vidwin_width, state_vidwin_height));
} // end of PostResize()

/*
**
** Command: off
**
*/
static int offFunc(int argc, char *argv[])
{
  return(0);
}



/*
**
** Command: full
**
*/
static int fullFunc(int argc, char *argv[])
{
//  VAPIOverlayDisable();
  return(0);
}



/*
**
** Command: position x y
**
*/
static int positionFunc(int argc, char *argv[])
{
  int result;

  state_vidwin_xpos = atoi(argv[0]);
  state_vidwin_ypos = atoi(argv[1]);
  result = PostResize(ERRORBASE_VIDWIN + 2);
//  result = VAPIOverlayPosition(state_vidwin_xpos,
//			       state_vidwin_ypos,
//			       state_vidwin_width,
//			       state_vidwin_height);
  return(result);
}



/*
**
** Command: size width height
**
*/
static int sizeFunc(int argc, char *argv[])
{
  int result;

  state_vidwin_width = atoi(argv[0]);
  state_vidwin_height = atoi(argv[1]);
  result = PostResize(ERRORBASE_VIDWIN + 4);
//  result = VAPIOverlayPosition(state_vidwin_xpos,
//			       state_vidwin_ypos,
//			       state_vidwin_width,
//			       state_vidwin_height);
  return(result);
}



/*
**
** Command: scale percent
**
*/
static int scaleFunc(int argc, char *argv[])
{
  double scaler;
  int result;

  scaler = ((double)atoi(argv[0])) / 100.0;
  state_vidwin_width = (int)(((double)state_vidwin_width) * scaler);
  state_vidwin_height = (int)(((double)state_vidwin_height) * scaler);
  result = PostResize(ERRORBASE_VIDWIN + 6);
// result = VAPIOverlayPosition(state_vidwin_xpos,
//			       state_vidwin_ypos,
//			       state_vidwin_width,
//			       state_vidwin_height);
  return(result);
}



/*
**
** Command: info
**
*/
static int infoFunc(int argc, char *argv[])
{
  printf("VidWin: xpos:%d ypos:%d width:%d height:%d\n",
	 state_vidwin_xpos,
	 state_vidwin_ypos,
	 state_vidwin_width,
	 state_vidwin_height);
  return(0);
}



/*
**
** Command: debug
**
*/
static int debugFunc(int argc, char *argv[])
{
  int winX,winY,winW,winH,ovrX,ovrY,ovrW,ovrH,bufW,bufH;

  winX = atoi(argv[0]);
  winY = atoi(argv[1]);
  winW = atoi(argv[2]);
  winH = atoi(argv[3]);
  ovrX = atoi(argv[4]);
  ovrY = atoi(argv[5]);
  ovrW = atoi(argv[6]);
  ovrH = atoi(argv[7]);
  bufW = atoi(argv[8]);
  bufH = atoi(argv[9]);
  return(debugOverlay(winX,winY,winW,winH,ovrX,ovrY,ovrW,ovrH,bufW,bufH));
}



/*
**
** Command List
**
*/
static CmdEntry cmdlist[] =
{
  "off",offFunc,
  "full",fullFunc,
  "position",positionFunc,
  "size",sizeFunc,
  "scale",scaleFunc,
  "info",infoFunc,
  "debug",debugFunc,
  NULL,NULL
};



/*
**
** This function will initialize the vidwin module and prepare it
** for subsequent operations.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginStartup(void)
{
  return(0);
}



/*
**
** This function will closeout the vidwin module and shut it down.
**
** If successful, '0' is returned. If an error occurs during function
** exeuction, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginShutdown(void)
{
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginName(char *result)
{
  strcpy(result,plugin_name);
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginVersion(char *result)
{
  strcpy(result,plugin_version);
  return(0);
}



/*
**
** This function will return the subsystem identification string
** for the vidwin module ("vidwin").
**
** The "result" parameter is a pointer to the buufer where the
** subsystem identification string will be stored.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginKeyword(char *result)
{
  strcpy(result,plugin_keyword);
  return(0);
}



/*
**
** This function will display the commands available for the vidwin
** module.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginHelp(void)
{
  printf("   %s off\n",plugin_keyword);
  printf("   %s full\n",plugin_keyword);
  printf("   %s position x y\n",plugin_keyword);
  printf("   %s size w h\n",plugin_keyword);
  printf("   %s scale n\n",plugin_keyword);
  printf("   %s info\n",plugin_keyword);
  return(0);
}



/*
**
** This function will execute the specified vidwin module command.
**
** The "argc" parameter specifies the number of string parameters
** found in the argv array. The "argv" parameter is an array of
** pointers to the command and it's parameters.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginCommand(int argc, char *argv[])
{
  int count,result;

  
  VMW_AVS_VideoInit();
//  VAPIOverlaySelect();
  
  // scan the list of commands for a match
  count = 0;
  while(1)
  {
    // check for end of list (indicates unknown command)
    if(cmdlist[count].cmd == NULL)
    {
      result = ERRORBASE_TUNER + 5;
      break;
    }

    // check for command match
    if(strcmp(cmdlist[count].cmd,argv[0]) == 0)
    {
      result = (cmdlist[count].func)(argc - 1,&argv[1]);
      break;
    }

    // bump to next command
    ++count;
  }

  VMW_AVS_VideoClose();
  
  // return result and exit
  return(result);
}
